@@ -52,6 +52,7 @@
 #include "dir.h"
 #include "growbuf.h"
 #include "outputlist.h"
+#include "moduledef.h"
 
 // no debug info
 #define Docbook_DB(x) do {} while(0)
@@ -79,11 +80,11 @@ inline void writeDocbookString(TextStream &t,const QCString &s)
   t << convertToDocBook(s);
 }
 
-inline void writeDocbookCodeString(TextStream &t,const QCString &str, int &col)
+inline void writeDocbookCodeString(TextStream &t,const QCString &str, size_t &col)
 {
   if (str.isEmpty()) return;
   const char *s = str.data();
-  char c;
+  char c=0;
   while ((c=*s++))
   {
     switch(c)
@@ -186,7 +187,7 @@ void DocbookCodeGenerator::writeTooltip(const QCString &, const DocLinkInfo &, c
   Docbook_DB(("(writeToolTip)\n"));
 }
 
-void DocbookCodeGenerator::startCodeLine(bool)
+void DocbookCodeGenerator::startCodeLine(int)
 {
   Docbook_DB(("(startCodeLine)\n"));
   m_insideCodeLine=TRUE;
@@ -198,8 +199,8 @@ void DocbookCodeGenerator::endCodeLine()
   if (m_insideCodeLine) *m_t << "\n";
   Docbook_DB(("(endCodeLine)\n"));
   m_lineNumber = -1;
-  m_refId.resize(0);
-  m_external.resize(0);
+  m_refId.clear();
+  m_external.clear();
   m_insideCodeLine=FALSE;
 }
 
@@ -283,10 +284,10 @@ DB_GEN_C
   m_codeGen = m_codeList->add<DocbookCodeGenerator>(&m_t);
 }
 
-DocbookGenerator::DocbookGenerator(const DocbookGenerator &og) : OutputGenerator(og.m_dir)
+DocbookGenerator::DocbookGenerator(const DocbookGenerator &og) : OutputGenerator(og.m_dir), OutputGenIntf()
 {
   m_codeList         = std::make_unique<OutputCodeList>(*og.m_codeList);
-  m_codeGen          = m_codeList->get<DocbookCodeGenerator>();
+  m_codeGen          = m_codeList->get<DocbookCodeGenerator>(OutputType::Docbook);
   m_codeGen->setTextStream(&m_t);
   m_denseText        = og.m_denseText;
   m_inGroup          = og.m_inGroup;
@@ -304,9 +305,9 @@ DocbookGenerator &DocbookGenerator::operator=(const DocbookGenerator &og)
 {
   if (this!=&og)
   {
-    m_dir          = og.m_dir;
-    m_codeList     = std::make_unique<OutputCodeList>(*og.m_codeList);
-    m_codeGen      = m_codeList->get<DocbookCodeGenerator>();
+    m_dir              = og.m_dir;
+    m_codeList         = std::make_unique<OutputCodeList>(*og.m_codeList);
+    m_codeGen          = m_codeList->get<DocbookCodeGenerator>(OutputType::Docbook);
     m_codeGen->setTextStream(&m_t);
     m_denseText        = og.m_denseText;
     m_inGroup          = og.m_inGroup;
@@ -322,31 +323,11 @@ DocbookGenerator &DocbookGenerator::operator=(const DocbookGenerator &og)
   return *this;
 }
 
-DocbookGenerator::DocbookGenerator(DocbookGenerator &&og)
-  : OutputGenerator(std::move(og))
-{
-  m_codeList         = std::exchange(og.m_codeList,std::unique_ptr<OutputCodeList>());
-  m_codeGen          = m_codeList->get<DocbookCodeGenerator>();
-  m_codeGen->setTextStream(&m_t);
-  m_denseText        = std::exchange(og.m_denseText,false);
-  m_inGroup          = std::exchange(og.m_inGroup,false);
-  m_levelListItem    = std::exchange(og.m_levelListItem,0);
-  m_inListItem       = std::exchange(og.m_inListItem,std::array<bool,20>());
-  m_inSimpleSect     = std::exchange(og.m_inSimpleSect,std::array<bool,20>());
-  m_descTable        = std::exchange(og.m_descTable,false);
-  m_simpleTable      = std::exchange(og.m_simpleTable,false);
-  m_inLevel          = std::exchange(og.m_inLevel,-1);
-  m_firstMember      = std::exchange(og.m_firstMember,false);
-  m_openSectionCount = std::exchange(og.m_openSectionCount,0);
-}
-
-DocbookGenerator::~DocbookGenerator()
-{
-}
+DocbookGenerator::~DocbookGenerator() = default;
 
 void DocbookGenerator::addCodeGen(OutputCodeList &list)
 {
-  list.add(OutputCodeList::OutputCodeVariant(DocbookCodeGeneratorDefer(m_codeGen)));
+  list.add<DocbookCodeGeneratorDefer>(m_codeGen);
 }
 
 void DocbookGenerator::init()
@@ -392,7 +373,7 @@ DB_GEN_C
   m_codeGen->setSourceFileName(stripPath(fileName));
   m_pageLinks = QCString();
 
-  m_t << "<?xml version='1.0' encoding='UTF-8' standalone='no'?>\n";;
+  m_t << "<?xml version='1.0' encoding='UTF-8' standalone='no'?>\n";
   m_t << "<" << fileType << " xmlns=\"http://docbook.org/ns/docbook\" version=\"5.0\" xmlns:xlink=\"http://www.w3.org/1999/xlink\"";
   if (!pageName.isEmpty()) m_t << " xml:id=\"_" <<  stripPath(pageName) << "\"";
   m_t << " xml:lang=\"" << theTranslator->trISOLang() << "\"";
@@ -444,6 +425,9 @@ DB_GEN_C2("IndexSection " << is)
     case IndexSection::isModuleIndex:
       //Module Index\n"
       break;
+    case IndexSection::isTopicIndex:
+      //Module Index\n"
+      break;
     case IndexSection::isDirIndex:
       //Directory Index\n"
       break;
@@ -469,6 +453,10 @@ DB_GEN_C2("IndexSection " << is)
       m_t << "<chapter>\n";
       m_t << "    <title>";
       break;
+    case IndexSection::isTopicDocumentation:
+      m_t << "<chapter>\n";
+      m_t << "    <title>";
+      break;
     case IndexSection::isDirDocumentation:
       m_t << "<chapter>\n";
       m_t << "    <title>";
@@ -522,6 +510,9 @@ DB_GEN_C2("IndexSection " << is)
     case IndexSection::isModuleIndex:
       //m_t << "</chapter>\n";
       break;
+    case IndexSection::isTopicIndex:
+      //m_t << "</chapter>\n";
+      break;
     case IndexSection::isDirIndex:
       //m_t << "<xi:include href=\"dirs.xml\" xmlns:xi=\"http://www.w3.org/2001/XInclude\"/>";
       //m_t << "</chapter>\n";
@@ -549,7 +540,7 @@ DB_GEN_C2("IndexSection " << is)
       //m_t << "<xi:include href=\"pages.xml\" xmlns:xi=\"http://www.w3.org/2001/XInclude\"/>";
       //m_t << "</chapter>\n";
       break;
-    case IndexSection::isModuleDocumentation:
+    case IndexSection::isTopicDocumentation:
       {
         m_t << "</title>\n";
         for (const auto &gd : *Doxygen::groupLinkedMap)
@@ -562,6 +553,20 @@ DB_GEN_C2("IndexSection " << is)
       }
       m_t << "</chapter>\n";
       break;
+    case IndexSection::isModuleDocumentation:
+      {
+        m_t << "</title>\n";
+        for (const auto &mod : ModuleManager::instance().modules())
+        {
+          if (!mod->isReference() && mod->isPrimaryInterface())
+          {
+            writePageLink(mod->getOutputFileBase(), TRUE);
+          }
+        }
+      }
+      m_t << "</chapter>\n";
+      break;
+      break;
     case IndexSection::isDirDocumentation:
       {
         m_t << "</title>\n";
@@ -607,7 +612,7 @@ DB_GEN_C2("IndexSection " << is)
         for (const auto &cd : *Doxygen::classLinkedMap)
         {
           if (cd->isLinkableInProject() &&
-              cd->templateMaster()==0 &&
+              cd->templateMaster()==nullptr &&
              !cd->isEmbeddedInOuterScope() &&
              !cd->isAlias()
              )
@@ -911,6 +916,10 @@ void DocbookGenerator::endDoxyAnchor(const QCString &,const QCString &)
 {
 DB_GEN_C
 }
+void DocbookGenerator::addLabel(const QCString &,const QCString &)
+{
+DB_GEN_C
+}
 void DocbookGenerator::startMemberDocName(bool)
 {
 DB_GEN_C
@@ -1029,19 +1038,46 @@ DB_GEN_C
   }
   m_t << " ";
 }
+
 void DocbookGenerator::startParameterName(bool)
 {
 DB_GEN_C
   m_t << " ";
 }
-void DocbookGenerator::endParameterName(bool last,bool /*emptyList*/,bool closeBracket)
+
+void DocbookGenerator::endParameterName()
 {
 DB_GEN_C
-  if (last)
+}
+
+void DocbookGenerator::startParameterExtra()
+{
+DB_GEN_C
+}
+
+void DocbookGenerator::endParameterExtra(bool last,bool /*emptyList*/,bool closeBracket)
+{
+DB_GEN_C
+  if (last && closeBracket)
   {
-    if (closeBracket) m_t << ")";
+    m_t << ")";
   }
 }
+
+
+void DocbookGenerator::startParameterDefVal(const char *sep)
+{
+DB_GEN_C
+  m_t << sep;
+  if (!m_denseText) m_t << "<computeroutput>";
+}
+
+void DocbookGenerator::endParameterDefVal()
+{
+DB_GEN_C
+  if (!m_denseText) m_t << "</computeroutput>\n";
+}
+
 void DocbookGenerator::startMemberTemplateParams()
 {
 DB_GEN_C
@@ -1074,7 +1110,7 @@ DB_GEN_C
 void DocbookGenerator::startMemberDocSimple(bool isEnum)
 {
 DB_GEN_C
-  int ncols;
+  int ncols=0;
   QCString title;
   if (isEnum)
   {
@@ -1142,17 +1178,17 @@ DB_GEN_C
   m_t << "</entry></row>\n";
 }
 
-void DocbookGenerator::startDescTable(const QCString &title)
+void DocbookGenerator::startDescTable(const QCString &title,const bool hasInits)
 {
 DB_GEN_C
-  int ncols = 2;
+  int ncols = (hasInits?3:2);
   m_t << "<informaltable frame=\"all\">\n";
   if (!title.isEmpty()) m_t << "<title>" << convertToDocBook(title) << "</title>\n";
   m_t << "    <tgroup cols=\"" << ncols << "\" align=\"left\" colsep=\"1\" rowsep=\"1\">\n";
-  for (int i = 0; i < ncols; i++)
-  {
-    m_t << "      <colspec colname='c" << i+1 << "'/>\n";
-  }
+  int i = 1;
+  m_t << "      <colspec colname='c" << i++ << "'/>\n";
+  if (hasInits) m_t << "      <colspec colname='c" << i++ << "' align='right'/>\n";
+  m_t << "      <colspec colname='c" << i++ << "'/>\n";
   m_t << "<tbody>\n";
   m_descTable = TRUE;
 }
@@ -1189,6 +1225,17 @@ void DocbookGenerator::endDescTableTitle()
 DB_GEN_C
 }
 
+void DocbookGenerator::startDescTableInit()
+{
+DB_GEN_C
+  m_t << "</entry><entry>";
+}
+
+void DocbookGenerator::endDescTableInit()
+{
+DB_GEN_C
+}
+
 void DocbookGenerator::startDescTableData()
 {
 DB_GEN_C
@@ -1207,7 +1254,7 @@ DB_GEN_C
 void DocbookGenerator::endGroupCollaboration(DotGroupCollaboration &g)
 {
 DB_GEN_C
-  g.writeGraph(m_t,GOF_BITMAP,EOF_DocBook,dir(),fileName(),relPath,FALSE);
+  g.writeGraph(m_t,GraphOutputFormat::BITMAP,EmbeddedOutputFormat::DocBook,dir(),fileName(),relPath,FALSE);
 }
 void DocbookGenerator::startDotGraph()
 {
@@ -1216,7 +1263,7 @@ DB_GEN_C
 void DocbookGenerator::endDotGraph(DotClassGraph &g)
 {
 DB_GEN_C
-  g.writeGraph(m_t,GOF_BITMAP,EOF_DocBook,dir(),fileName(),relPath,TRUE,FALSE);
+  g.writeGraph(m_t,GraphOutputFormat::BITMAP,EmbeddedOutputFormat::DocBook,dir(),fileName(),relPath,TRUE,FALSE);
 }
 void DocbookGenerator::startInclDepGraph()
 {
@@ -1225,7 +1272,7 @@ DB_GEN_C
 void DocbookGenerator::endInclDepGraph(DotInclDepGraph &g)
 {
 DB_GEN_C
-  QCString fn = g.writeGraph(m_t,GOF_BITMAP,EOF_DocBook,dir(),fileName(),relPath,FALSE);
+  QCString fn = g.writeGraph(m_t,GraphOutputFormat::BITMAP,EmbeddedOutputFormat::DocBook,dir(),fileName(),relPath,FALSE);
 }
 void DocbookGenerator::startCallGraph()
 {
@@ -1234,7 +1281,7 @@ DB_GEN_C
 void DocbookGenerator::endCallGraph(DotCallGraph &g)
 {
 DB_GEN_C
-  QCString fn = g.writeGraph(m_t,GOF_BITMAP,EOF_DocBook,dir(),fileName(),relPath,FALSE);
+  QCString fn = g.writeGraph(m_t,GraphOutputFormat::BITMAP,EmbeddedOutputFormat::DocBook,dir(),fileName(),relPath,FALSE);
 }
 void DocbookGenerator::startDirDepGraph()
 {
@@ -1243,7 +1290,7 @@ DB_GEN_C
 void DocbookGenerator::endDirDepGraph(DotDirDeps &g)
 {
 DB_GEN_C
-  QCString fn = g.writeGraph(m_t,GOF_BITMAP,EOF_DocBook,dir(),fileName(),relPath,FALSE);
+  QCString fn = g.writeGraph(m_t,GraphOutputFormat::BITMAP,EmbeddedOutputFormat::DocBook,dir(),fileName(),relPath,FALSE);
 }
 void DocbookGenerator::startMemberDocList()
 {
@@ -1331,26 +1378,26 @@ void DocbookGenerator::writeLocalToc(const SectionRefs &sectionRefs,const LocalT
   {
     m_t << "    <toc>\n";
     m_t << "    <title>" << theTranslator->trRTFTableOfContents() << "</title>\n";
-    int level=1,l;
+    int level=1;
     int maxLevel = localToc.docbookLevel();
     BoolVector inLi(maxLevel+1,false);
     for (const SectionInfo *si : sectionRefs)
     {
       SectionType type = si->type();
-      if (isSection(type))
+      if (type.isSection())
       {
         //printf("  level=%d title=%s\n",level,qPrint(si->title));
-        int nextLevel = static_cast<int>(type);
+        int nextLevel = type.level();
         if (nextLevel>level)
         {
-          for (l=level;l<nextLevel;l++)
+          for (int l=level;l<nextLevel;l++)
           {
             if (l < maxLevel) m_t << "    <tocdiv>\n";
           }
         }
         else if (nextLevel<level)
         {
-          for (l=level;l>nextLevel;l--)
+          for (int l=level;l>nextLevel;l--)
           {
             inLi[l]=FALSE;
             if (l <= maxLevel) m_t << "    </tocdiv>\n";
@@ -1387,10 +1434,10 @@ QCString convertToDocBook(const QCString &s, const bool retainNewline)
 {
   if (s.isEmpty()) return s;
   GrowBuf growBuf;
-  const char *q;
-  int cnt;
-  const char *p=s.data();
-  char c;
+  const char *p = s.data();
+  const char *q = nullptr;
+  int cnt = 0;
+  char c = 0;
   while ((c=*p++))
   {
     switch (c)
