@@ -42,7 +42,7 @@
 
 static const int NUM_HTML_LIST_TYPES = 4;
 static const char types[][NUM_HTML_LIST_TYPES] = {"1", "a", "i", "A"};
-enum contexts_t
+enum class contexts_t
 {
     NONE,      // 0
     STARTLI,   // 1
@@ -55,18 +55,24 @@ enum contexts_t
     INTERDD,   // 8
     INTERTD    // 9
 };
-static const char *contexts[10] =
-{ "",          // 0
-  "startli",   // 1
-  "startdd",   // 2
-  "endli",     // 3
-  "enddd",     // 4
-  "starttd",   // 5
-  "endtd",     // 6
-  "interli",   // 7
-  "interdd",   // 8
-  "intertd"    // 9
-};
+
+static constexpr const char *contexts(contexts_t type)
+{
+  switch (type)
+  {
+    case contexts_t::NONE:    return nullptr;
+    case contexts_t::STARTLI: return "startli";
+    case contexts_t::STARTDD: return "startdd";
+    case contexts_t::ENDLI:   return "endli";
+    case contexts_t::ENDDD:   return "enddd";
+    case contexts_t::STARTTD: return "starttd";
+    case contexts_t::ENDTD:   return "endtd";
+    case contexts_t::INTERLI: return "interli";
+    case contexts_t::INTERDD: return "interdd";
+    case contexts_t::INTERTD: return "intertd";
+    default:                  return nullptr;
+  }
+}
 static const char *hex="0123456789ABCDEF";
 
 static QCString convertIndexWordToAnchor(const QCString &word)
@@ -78,7 +84,7 @@ static QCString convertIndexWordToAnchor(const QCString &word)
   result += "_";
   cnt++;
   const char *str = word.data();
-  unsigned char c;
+  unsigned char c = 0;
   if (str)
   {
     while ((c = *str++))
@@ -87,8 +93,7 @@ static QCString convertIndexWordToAnchor(const QCString &word)
           (c >= 'A' && c <= 'Z') || // ALPHA
           (c >= '0' && c <= '9') || // DIGIT
           c == '-' ||
-          c == '.' ||
-          c == '_'
+          c == '.'
          )
       {
         result += c;
@@ -96,7 +101,7 @@ static QCString convertIndexWordToAnchor(const QCString &word)
       else
       {
         char enc[4];
-        enc[0] = ':';
+        enc[0] = '_';
         enc[1] = hex[(c & 0xf0) >> 4];
         enc[2] = hex[c & 0xf];
         enc[3] = 0;
@@ -237,7 +242,7 @@ static void mergeHtmlAttributes(const HtmlAttribList &attribs, HtmlAttribList &m
   }
 }
 
-static QCString htmlAttribsToString(const HtmlAttribList &attribs, QCString *pAltValue = 0)
+static QCString htmlAttribsToString(const HtmlAttribList &attribs, QCString *pAltValue = nullptr)
 {
   QCString result;
   for (const auto &att : attribs)
@@ -278,6 +283,29 @@ static QCString htmlAttribsToString(const HtmlAttribList &attribs, QCString *pAl
   return result;
 }
 
+static QCString makeShortName(const QCString &name)
+{
+  QCString shortName = name;
+  int i = shortName.findRev('/');
+  if (i!=-1)
+  {
+    shortName=shortName.mid(i+1);
+  }
+  return shortName;
+}
+
+static QCString makeBaseName(const QCString &name)
+{
+  QCString baseName = makeShortName(name);
+  int i=baseName.find('.');
+  if (i!=-1)
+  {
+    baseName=baseName.left(i);
+  }
+  return baseName;
+}
+
+
 //-------------------------------------------------------------------------
 
 HtmlDocVisitor::HtmlDocVisitor(TextStream &t,OutputCodeList &ci,
@@ -557,11 +585,11 @@ void HtmlDocVisitor::operator()(const DocVerbatim &s)
                                         langExt,
                                         s.isExample(),
                                         s.exampleFile(),
-                                        0,     // fileDef
+                                        nullptr,     // fileDef
                                         -1,    // startLine
                                         -1,    // endLine
                                         FALSE, // inlineFragment
-                                        0,     // memberDef
+                                        nullptr,     // memberDef
                                         TRUE,  // show line numbers
                                         m_ctx  // search context
                                        );
@@ -601,7 +629,7 @@ void HtmlDocVisitor::operator()(const DocVerbatim &s)
     case DocVerbatim::Dot:
       {
         static int dotindex = 1;
-        QCString fileName(4096);
+        QCString fileName(4096, QCString::ExplicitSize);
 
         forceEndParagraph(s);
         fileName.sprintf("%s%d%s",
@@ -635,7 +663,7 @@ void HtmlDocVisitor::operator()(const DocVerbatim &s)
         forceEndParagraph(s);
 
         static int mscindex = 1;
-        QCString baseName(4096);
+        QCString baseName(4096, QCString::ExplicitSize);
 
         baseName.sprintf("%s%d",
             qPrint(Config_getString(HTML_OUTPUT)+"/inline_mscgraph_"),
@@ -709,11 +737,11 @@ void HtmlDocVisitor::operator()(const DocInclude &inc)
                                         langExt,
                                         inc.isExample(),
                                         inc.exampleFile(),
-                                        0,     // fileDef
+                                        nullptr,     // fileDef
                                         -1,    // startLine
                                         -1,    // endLine
                                         TRUE,  // inlineFragment
-                                        0,     // memberDef
+                                        nullptr,     // memberDef
                                         FALSE, // show line numbers
                                         m_ctx  // search context
                                        );
@@ -736,7 +764,7 @@ void HtmlDocVisitor::operator()(const DocInclude &inc)
                                            -1,    // start line
                                            -1,    // end line
                                            FALSE, // inline fragment
-                                           0,     // memberDef
+                                           nullptr,     // memberDef
                                            TRUE,  // show line numbers
                                            m_ctx  // search context
                                            );
@@ -820,7 +848,7 @@ void HtmlDocVisitor::operator()(const DocIncOperator &op)
                                 op.line(),    // startLine
                                 -1,    // endLine
                                 FALSE, // inline fragment
-                                0,     // memberDef
+                                nullptr,     // memberDef
                                 op.showLineNo(),  // show line numbers
                                 m_ctx  // search context
                                );
@@ -1022,7 +1050,14 @@ void HtmlDocVisitor::operator()(const DocAutoList &l)
   }
   else
   {
-    m_t << "<ul>";
+    if (l.isCheckedList())
+    {
+      m_t << "<ul class=\"check\">";
+    }
+    else
+    {
+      m_t << "<ul>";
+    }
   }
   if (!l.isPreformatted()) m_t << "\n";
   visitChildren(l);
@@ -1041,7 +1076,19 @@ void HtmlDocVisitor::operator()(const DocAutoList &l)
 void HtmlDocVisitor::operator()(const DocAutoListItem &li)
 {
   if (m_hide) return;
-  m_t << "<li>";
+  switch (li.itemNumber())
+  {
+    case DocAutoList::Unchecked: // unchecked
+      m_t << "<li class=\"unchecked\">";
+      break;
+    case DocAutoList::Checked_x: // checked with x
+    case DocAutoList::Checked_X: // checked with X
+      m_t << "<li class=\"checked\">";
+      break;
+    default:
+      m_t << "<li>";
+      break;
+  }
   visitChildren(li);
   m_t << "</li>";
   if (!li.isPreformatted()) m_t << "\n";
@@ -1076,7 +1123,7 @@ bool isSeparatedParagraph(const DocSimpleSect &parent,const DocPara &par)
   auto it = std::find_if(std::begin(nodes),std::end(nodes),[&par](const auto &n) { return holds_value(&par,n); });
   if (it==std::end(nodes)) return FALSE;
   size_t count = parent.children().size();
-  auto isSeparator = [](auto &&it_) { return std::get_if<DocSimpleSectSep>(&(*it_))!=0; };
+  auto isSeparator = [](auto &&it_) { return std::get_if<DocSimpleSectSep>(&(*it_))!=nullptr; };
   if (count>1 && it==std::begin(nodes)) // it points to first node
   {
     return isSeparator(std::next(it));
@@ -1092,9 +1139,9 @@ bool isSeparatedParagraph(const DocSimpleSect &parent,const DocPara &par)
   return false;
 }
 
-static int getParagraphContext(const DocPara &p,bool &isFirst,bool &isLast)
+static contexts_t getParagraphContext(const DocPara &p,bool &isFirst,bool &isLast)
 {
-  int t=0;
+  contexts_t t=contexts_t::NONE;
   isFirst=FALSE;
   isLast=FALSE;
   if (p.parent())
@@ -1104,28 +1151,28 @@ static int getParagraphContext(const DocPara &p,bool &isFirst,bool &isLast)
     {
       // hierarchy: node N -> para -> parblock -> para
       // adapt return value to kind of N
-      const DocNodeVariant *p3 = 0;
+      const DocNodeVariant *p3 = nullptr;
       if (::parent(p.parent()) && ::parent(::parent(p.parent())) )
       {
         p3 = ::parent(::parent(p.parent()));
       }
       isFirst=isFirstChildNode(parBlock,p);
       isLast =isLastChildNode (parBlock,p);
-      bool isLI = p3!=0 && holds_one_of_alternatives<DocHtmlListItem,DocSecRefItem>(*p3);
-      bool isDD = p3!=0 && holds_one_of_alternatives<DocHtmlDescData,DocXRefItem,DocSimpleSect>(*p3);
-      bool isTD = p3!=0 && holds_one_of_alternatives<DocHtmlCell,DocParamList>(*p3);
-      t=NONE;
+      bool isLI = p3!=nullptr && holds_one_of_alternatives<DocHtmlListItem,DocSecRefItem>(*p3);
+      bool isDD = p3!=nullptr && holds_one_of_alternatives<DocHtmlDescData,DocXRefItem,DocSimpleSect>(*p3);
+      bool isTD = p3!=nullptr && holds_one_of_alternatives<DocHtmlCell,DocParamList>(*p3);
+      t=contexts_t::NONE;
       if (isFirst)
       {
-        if (isLI) t=STARTLI; else if (isDD) t=STARTDD; else if (isTD) t=STARTTD;
+        if (isLI) t=contexts_t::STARTLI; else if (isDD) t=contexts_t::STARTDD; else if (isTD) t=contexts_t::STARTTD;
       }
       if (isLast)
       {
-        if (isLI) t=ENDLI;   else if (isDD) t=ENDDD;   else if (isTD) t=ENDTD;
+        if (isLI) t=contexts_t::ENDLI;   else if (isDD) t=contexts_t::ENDDD;   else if (isTD) t=contexts_t::ENDTD;
       }
       if (!isFirst && !isLast)
       {
-        if (isLI) t=INTERLI; else if (isDD) t=INTERDD; else if (isTD) t=INTERTD;
+        if (isLI) t=contexts_t::INTERLI; else if (isDD) t=contexts_t::INTERDD; else if (isTD) t=contexts_t::INTERTD;
       }
       return t;
     }
@@ -1134,7 +1181,7 @@ static int getParagraphContext(const DocPara &p,bool &isFirst,bool &isLast)
     {
       isFirst=isFirstChildNode(docAutoListItem,p);
       isLast =isLastChildNode (docAutoListItem,p);
-      t=STARTLI; // not used
+      t=contexts_t::STARTLI; // not used
       return t;
     }
     const auto docSimpleListItem = std::get_if<DocSimpleListItem>(p.parent());
@@ -1142,7 +1189,7 @@ static int getParagraphContext(const DocPara &p,bool &isFirst,bool &isLast)
     {
       isFirst=TRUE;
       isLast =TRUE;
-      t=STARTLI; // not used
+      t=contexts_t::STARTLI; // not used
       return t;
     }
     const auto docParamList = std::get_if<DocParamList>(p.parent());
@@ -1150,7 +1197,7 @@ static int getParagraphContext(const DocPara &p,bool &isFirst,bool &isLast)
     {
       isFirst=TRUE;
       isLast =TRUE;
-      t=STARTLI; // not used
+      t=contexts_t::STARTLI; // not used
       return t;
     }
     const auto docHtmlListItem = std::get_if<DocHtmlListItem>(p.parent());
@@ -1158,9 +1205,9 @@ static int getParagraphContext(const DocPara &p,bool &isFirst,bool &isLast)
     {
       isFirst=isFirstChildNode(docHtmlListItem,p);
       isLast =isLastChildNode (docHtmlListItem,p);
-      if (isFirst) t=STARTLI;
-      if (isLast)  t=ENDLI;
-      if (!isFirst && !isLast) t = INTERLI;
+      if (isFirst) t=contexts_t::STARTLI;
+      if (isLast)  t=contexts_t::ENDLI;
+      if (!isFirst && !isLast) t = contexts_t::INTERLI;
       return t;
     }
     const auto docSecRefItem = std::get_if<DocSecRefItem>(p.parent());
@@ -1168,9 +1215,9 @@ static int getParagraphContext(const DocPara &p,bool &isFirst,bool &isLast)
     {
       isFirst=isFirstChildNode(docSecRefItem,p);
       isLast =isLastChildNode (docSecRefItem,p);
-      if (isFirst) t=STARTLI;
-      if (isLast)  t=ENDLI;
-      if (!isFirst && !isLast) t = INTERLI;
+      if (isFirst) t=contexts_t::STARTLI;
+      if (isLast)  t=contexts_t::ENDLI;
+      if (!isFirst && !isLast) t = contexts_t::INTERLI;
       return t;
     }
     const auto docHtmlDescData = std::get_if<DocHtmlDescData>(p.parent());
@@ -1178,9 +1225,9 @@ static int getParagraphContext(const DocPara &p,bool &isFirst,bool &isLast)
     {
       isFirst=isFirstChildNode(docHtmlDescData,p);
       isLast =isLastChildNode (docHtmlDescData,p);
-      if (isFirst) t=STARTDD;
-      if (isLast)  t=ENDDD;
-      if (!isFirst && !isLast) t = INTERDD;
+      if (isFirst) t=contexts_t::STARTDD;
+      if (isLast)  t=contexts_t::ENDDD;
+      if (!isFirst && !isLast) t = contexts_t::INTERDD;
       return t;
     }
     const auto docXRefItem = std::get_if<DocXRefItem>(p.parent());
@@ -1188,9 +1235,9 @@ static int getParagraphContext(const DocPara &p,bool &isFirst,bool &isLast)
     {
       isFirst=isFirstChildNode(docXRefItem,p);
       isLast =isLastChildNode (docXRefItem,p);
-      if (isFirst) t=STARTDD;
-      if (isLast)  t=ENDDD;
-      if (!isFirst && !isLast) t = INTERDD;
+      if (isFirst) t=contexts_t::STARTDD;
+      if (isLast)  t=contexts_t::ENDDD;
+      if (!isFirst && !isLast) t = contexts_t::INTERDD;
       return t;
     }
     const auto docSimpleSect = std::get_if<DocSimpleSect>(p.parent());
@@ -1198,8 +1245,8 @@ static int getParagraphContext(const DocPara &p,bool &isFirst,bool &isLast)
     {
       isFirst=isFirstChildNode(docSimpleSect,p);
       isLast =isLastChildNode (docSimpleSect,p);
-      if (isFirst) t=STARTDD;
-      if (isLast)  t=ENDDD;
+      if (isFirst) t=contexts_t::STARTDD;
+      if (isLast)  t=contexts_t::ENDDD;
       if (isSeparatedParagraph(*docSimpleSect,p))
         // if the paragraph is enclosed with separators it will
         // be included in <dd>..</dd> so avoid addition paragraph
@@ -1207,7 +1254,7 @@ static int getParagraphContext(const DocPara &p,bool &isFirst,bool &isLast)
       {
         isFirst=isLast=TRUE;
       }
-      if (!isFirst && !isLast) t = INTERDD;
+      if (!isFirst && !isLast) t = contexts_t::INTERDD;
       return t;
     }
     const auto docHtmlCell = std::get_if<DocHtmlCell>(p.parent());
@@ -1215,9 +1262,9 @@ static int getParagraphContext(const DocPara &p,bool &isFirst,bool &isLast)
     {
       isFirst=isFirstChildNode(docHtmlCell,p);
       isLast =isLastChildNode (docHtmlCell,p);
-      if (isFirst) t=STARTTD;
-      if (isLast)  t=ENDTD;
-      if (!isFirst && !isLast) t = INTERTD;
+      if (isFirst) t=contexts_t::STARTTD;
+      if (isLast)  t=contexts_t::ENDTD;
+      if (!isFirst && !isLast) t = contexts_t::INTERTD;
       return t;
     }
   }
@@ -1286,10 +1333,9 @@ void HtmlDocVisitor::operator()(const DocPara &p)
   // check if this paragraph is the first or last or intermediate child of a <li> or <dd>.
   // this allows us to mark the tag with a special class so we can
   // fix the otherwise ugly spacing.
-  int t;
-  bool isFirst;
-  bool isLast;
-  t = getParagraphContext(p,isFirst,isLast);
+  bool isFirst = false;
+  bool isLast  = false;
+  contexts_t t = getParagraphContext(p,isFirst,isLast);
   //printf("startPara first=%d last=%d\n",isFirst,isLast);
   if (isFirst && isLast) needsTagBefore=FALSE;
 
@@ -1297,8 +1343,8 @@ void HtmlDocVisitor::operator()(const DocPara &p)
   // write the paragraph tag (if needed)
   if (needsTagBefore)
   {
-    if (strlen(contexts[t]))
-      m_t << "<p class=\"" << contexts[t] << "\"" << htmlAttribsToString(p.attribs()) << ">";
+    if (contexts(t))
+      m_t << "<p class=\"" << contexts(t) << "\"" << htmlAttribsToString(p.attribs()) << ">";
     else
       m_t << "<p" << htmlAttribsToString(p.attribs()) << ">";
   }
@@ -1388,6 +1434,8 @@ void HtmlDocVisitor::operator()(const DocSimpleSect &s)
       m_t << theTranslator->trRemarks(); break;
     case DocSimpleSect::Attention:
       m_t << theTranslator->trAttention(); break;
+    case DocSimpleSect::Important:
+      m_t << theTranslator->trImportant(); break;
     case DocSimpleSect::User: break;
     case DocSimpleSect::Rcs: break;
     case DocSimpleSect::Unknown:  break;
@@ -1643,12 +1691,7 @@ void HtmlDocVisitor::operator()(const DocImage &img)
       forceEndParagraph(img);
     }
     if (m_hide) return;
-    QCString baseName=img.name();
-    int i;
-    if ((i=baseName.findRev('/'))!=-1 || (i=baseName.findRev('\\'))!=-1)
-    {
-      baseName=baseName.right(baseName.length()-i-1);
-    }
+    QCString baseName=makeShortName(img.name());
     if (!inlineImage) m_t << "<div class=\"image\">\n";
     QCString sizeAttribs;
     if (!img.width().isEmpty())
@@ -2016,6 +2059,7 @@ void HtmlDocVisitor::operator()(const DocVhdlFlow &vf)
     QCString fname=FlowChart::convertNameToFileName();
     m_t << "<p>";
     m_t << theTranslator->trFlowchart();
+    m_t << " ";
     m_t << "<a href=\"";
     m_t << fname;
     m_t << ".svg\">";
@@ -2044,10 +2088,9 @@ void HtmlDocVisitor::filter(const QCString &str, const bool retainNewline)
 {
   if (str.isEmpty()) return;
   const char *p=str.data();
-  char c;
   while (*p)
   {
-    c=*p++;
+    char c=*p++;
     switch(c)
     {
       case '\n': if(retainNewline) m_t << "<br/>"; m_t << c; break;
@@ -2083,10 +2126,9 @@ QCString HtmlDocVisitor::filterQuotedCdataAttr(const QCString &str)
   GrowBuf growBuf;
   if (str.isEmpty()) return str;
   const char *p=str.data();
-  char c;
   while (*p)
   {
-    c=*p++;
+    char c=*p++;
     switch(c)
     {
       case '&':  growBuf.addStr("&amp;"); break;
@@ -2160,41 +2202,23 @@ void HtmlDocVisitor::endLink()
 void HtmlDocVisitor::writeDotFile(const QCString &fn,const QCString &relPath,
                                   const QCString &context,const QCString &srcFile,int srcLine)
 {
-  QCString baseName=fn;
-  int i;
-  if ((i=baseName.findRev('/'))!=-1)
-  {
-    baseName=baseName.right(baseName.length()-i-1);
-  }
-  if ((i=baseName.find('.'))!=-1) // strip extension
-  {
-    baseName=baseName.left(i);
-  }
+  QCString baseName=makeBaseName(fn);
   baseName.prepend("dot_");
   QCString outDir = Config_getString(HTML_OUTPUT);
-  writeDotGraphFromFile(fn,outDir,baseName,GOF_BITMAP,srcFile,srcLine);
+  writeDotGraphFromFile(fn,outDir,baseName,GraphOutputFormat::BITMAP,srcFile,srcLine);
   writeDotImageMapFromFile(m_t,fn,outDir,relPath,baseName,context,-1,srcFile,srcLine);
 }
 
 void HtmlDocVisitor::writeMscFile(const QCString &fileName,const QCString &relPath,
                                   const QCString &context,const QCString &srcFile,int srcLine)
 {
-  QCString baseName=fileName;
-  int i;
-  if ((i=baseName.findRev('/'))!=-1) // strip path
-  {
-    baseName=baseName.right(baseName.length()-i-1);
-  }
-  if ((i=baseName.find('.'))!=-1) // strip extension
-  {
-    baseName=baseName.left(i);
-  }
+  QCString baseName=makeBaseName(fileName);
   baseName.prepend("msc_");
   QCString outDir = Config_getString(HTML_OUTPUT);
   QCString imgExt = getDotImageExtension();
-  MscOutputFormat mscFormat = MSC_BITMAP;
+  MscOutputFormat mscFormat = MscOutputFormat::BITMAP;
   if ("svg" == imgExt)
-    mscFormat = MSC_SVG;
+    mscFormat = MscOutputFormat::SVG;
   writeMscGraphFromFile(fileName,outDir,baseName,mscFormat,srcFile,srcLine);
   writeMscImageMapFromFile(m_t,fileName,outDir,relPath,baseName,context,mscFormat,srcFile,srcLine);
 }
@@ -2202,19 +2226,10 @@ void HtmlDocVisitor::writeMscFile(const QCString &fileName,const QCString &relPa
 void HtmlDocVisitor::writeDiaFile(const QCString &fileName, const QCString &relPath,
                                   const QCString &,const QCString &srcFile,int srcLine)
 {
-  QCString baseName=fileName;
-  int i;
-  if ((i=baseName.findRev('/'))!=-1) // strip path
-  {
-    baseName=baseName.right(baseName.length()-i-1);
-  }
-  if ((i=baseName.find('.'))!=-1) // strip extension
-  {
-    baseName=baseName.left(i);
-  }
+  QCString baseName=makeBaseName(fileName);
   baseName.prepend("dia_");
   QCString outDir = Config_getString(HTML_OUTPUT);
-  writeDiaGraphFromFile(fileName,outDir,baseName,DIA_BITMAP,srcFile,srcLine);
+  writeDiaGraphFromFile(fileName,outDir,baseName,DiaOutputFormat::BITMAP,srcFile,srcLine);
 
   m_t << "<img src=\"" << relPath << baseName << ".png" << "\" />\n";
 }
@@ -2222,16 +2237,7 @@ void HtmlDocVisitor::writeDiaFile(const QCString &fileName, const QCString &relP
 void HtmlDocVisitor::writePlantUMLFile(const QCString &fileName, const QCString &relPath,
                                        const QCString &,const QCString &/* srcFile */,int /* srcLine */)
 {
-  QCString baseName=fileName;
-  int i;
-  if ((i=baseName.findRev('/'))!=-1) // strip path
-  {
-    baseName=baseName.right(baseName.length()-i-1);
-  }
-  if ((i=baseName.findRev('.'))!=-1) // strip extension
-  {
-    baseName=baseName.left(i);
-  }
+  QCString baseName=makeBaseName(fileName);
   QCString outDir = Config_getString(HTML_OUTPUT);
   QCString imgExt = getDotImageExtension();
   if (imgExt=="svg")
@@ -2336,8 +2342,8 @@ void HtmlDocVisitor::forceEndParagraph(const Node &n)
       it = std::prev(it);
       styleOutsideParagraph=insideStyleChangeThatIsOutsideParagraph(para,it);
     }
-    bool isFirst;
-    bool isLast;
+    bool isFirst = false;
+    bool isLast = false;
     getParagraphContext(*para,isFirst,isLast);
     //printf("forceEnd first=%d last=%d styleOutsideParagraph=%d\n",isFirst,isLast,styleOutsideParagraph);
     if (isFirst && isLast) return;
@@ -2356,7 +2362,7 @@ template<class Node>
 void HtmlDocVisitor::forceStartParagraph(const Node &n)
 {
   //printf("> forceStartParagraph(%s)\n",docNodeName(n));
-  const DocPara *para=0;
+  const DocPara *para=nullptr;
   if (n.parent() && (para = std::get_if<DocPara>(n.parent()))) // if we are inside a paragraph
   {
     const DocNodeList &children = para->children();
@@ -2385,11 +2391,11 @@ void HtmlDocVisitor::forceStartParagraph(const Node &n)
       return; // only whitespace at the end!
     }
 
-    bool needsTag = TRUE;
-    bool isFirst;
-    bool isLast;
+    bool needsTag = true;
+    bool isFirst = false;
+    bool isLast = false;
     getParagraphContext(*para,isFirst,isLast);
-    if (isFirst && isLast) needsTag = FALSE;
+    if (isFirst && isLast) needsTag = false;
     //printf("forceStart first=%d last=%d needsTag=%d\n",isFirst,isLast,needsTag);
 
     if (needsTag) m_t << "<p>";
