@@ -1,6 +1,6 @@
 /******************************************************************************
  *
- * Copyright (C) 1997-2021 by Dimitri van Heesch.
+ * Copyright (C) 1997-2023 by Dimitri van Heesch.
  *
  * Permission to use, copy, modify, and distribute this software and its
  * documentation under the terms of the GNU General Public License is hereby
@@ -46,9 +46,13 @@
 #include "datetime.h"
 #include "portable.h"
 #include "outputlist.h"
+#include "moduledef.h"
 
 static QCString g_header;
 static QCString g_footer;
+static const SelectionMarkerInfo latexMarkerInfo = { '%', "%%BEGIN ",8 ,"%%END ",6, "",0 };
+
+static QCString substituteLatexKeywords(const QCString &str, const QCString &title);
 
 LatexCodeGenerator::LatexCodeGenerator(TextStream *t,const QCString &relPath,const QCString &sourceFileName)
   : m_t(t), m_relPath(relPath), m_sourceFileName(sourceFileName)
@@ -74,13 +78,11 @@ void LatexCodeGenerator::codify(const QCString &str)
   if (!str.isEmpty())
   {
     const char *p=str.data();
-    char c;
+    char c = 0;
     //char cs[5];
-    int spacesToNextTabStop;
     int tabSize = Config_getInt(TAB_SIZE);
-    static THREAD_LOCAL char *result = NULL;
+    static THREAD_LOCAL char *result = nullptr;
     static THREAD_LOCAL int lresult = 0;
-    int i;
     while ((c=*p))
     {
       switch(c)
@@ -99,51 +101,54 @@ void LatexCodeGenerator::codify(const QCString &str)
                    m_col++;
                    p++;
                    break;
-        case '\t': spacesToNextTabStop =
-                         tabSize - (m_col%tabSize);
-                   for (i = 0; i < spacesToNextTabStop; i++) *m_t << (m_doxyCodeLineOpen ? "\\ " : " ");
-                   m_col+=spacesToNextTabStop;
-                   p++;
+        case '\t': {
+                     int spacesToNextTabStop = tabSize - (m_col%tabSize);
+                     for (int i = 0; i < spacesToNextTabStop; i++) *m_t << (m_doxyCodeLineOpen ? "\\ " : " ");
+                     m_col+=spacesToNextTabStop;
+                     p++;
+                   }
                    break;
         case '\n': *m_t << '\n';
                    m_col=0;
                    p++;
                    break;
         default:
-                   i=0;
+                   {
+                     int i=0;
 
 #undef  COPYCHAR
 // helper macro to copy a single utf8 character, dealing with multibyte chars.
-#define COPYCHAR() do {                                           \
-                     int bytes = getUTF8CharNumBytes(c);          \
-                     if (lresult < (i + bytes + 1))               \
-                     {                                            \
-                       lresult += 512;                            \
-                       result = static_cast<char *>(realloc(result, lresult)); \
-                     }                                            \
-                     for (int j=0; j<bytes && *p; j++)            \
-                     {                                            \
-                       result[i++]=*p++;                          \
-                     }                                            \
-                     m_col++;                                     \
-                   } while(0)
+#define COPYCHAR() do {                                             \
+                       int bytes = getUTF8CharNumBytes(c);          \
+                       if (lresult < (i + bytes + 1))               \
+                       {                                            \
+                         lresult += 512;                            \
+                         result = static_cast<char *>(realloc(result, lresult)); \
+                       }                                            \
+                       for (int j=0; j<bytes && *p; j++)            \
+                       {                                            \
+                         result[i++]=*p++;                          \
+                       }                                            \
+                       m_col++;                                     \
+                     } while(0)
 
-                   // gather characters until we find whitespace or another special character
-                   COPYCHAR();
-                   while ((c=*p) &&
-                          c!=0x0c && c!='\t' && c!='\n' && c!=' ' && c!='^'
-                         )
-                   {
+                     // gather characters until we find whitespace or another special character
                      COPYCHAR();
+                     while ((c=*p) &&
+                            c!=0x0c && c!='\t' && c!='\n' && c!=' ' && c!='^'
+                           )
+                     {
+                       COPYCHAR();
+                     }
+                     result[i]=0; // add terminator
+                     filterLatexString(*m_t,result,
+                                       m_insideTabbing, // insideTabbing
+                                       true,  // insidePre
+                                       false, // insideItem
+                                       m_usedTableLevel>0, // insideTable
+                                       false  // keepSpaces
+                                      );
                    }
-                   result[i]=0; // add terminator
-                   filterLatexString(*m_t,result,
-                                     m_insideTabbing, // insideTabbing
-                                     true,  // insidePre
-                                     false, // insideItem
-                                     m_usedTableLevel>0, // insideTable
-                                     false  // keepSpaces
-                                    );
                    break;
       }
     }
@@ -158,7 +163,7 @@ void LatexCodeGenerator::writeCodeLink(CodeSymbolType,
 {
   bool pdfHyperlinks = Config_getBool(PDF_HYPERLINKS);
   bool usePDFLatex   = Config_getBool(USE_PDFLATEX);
-  uint32_t l = name.length();
+  size_t l = name.length();
   if (ref.isEmpty() && usePDFLatex && pdfHyperlinks)
   {
     *m_t << "\\mbox{\\hyperlink{";
@@ -222,7 +227,7 @@ void LatexCodeGenerator::writeLineNumber(const QCString &ref,const QCString &fil
 }
 
 
-void LatexCodeGenerator::startCodeLine(bool)
+void LatexCodeGenerator::startCodeLine(int)
 {
   m_col=0;
   if (!m_doxyCodeLineOpen)
@@ -275,10 +280,10 @@ LatexGenerator::LatexGenerator()
   m_codeGen = m_codeList->add<LatexCodeGenerator>(&m_t);
 }
 
-LatexGenerator::LatexGenerator(const LatexGenerator &og) : OutputGenerator(og.m_dir)
+LatexGenerator::LatexGenerator(const LatexGenerator &og) : OutputGenerator(og.m_dir), OutputGenIntf()
 {
   m_codeList           = std::make_unique<OutputCodeList>(*og.m_codeList);
-  m_codeGen            = m_codeList->get<LatexCodeGenerator>();
+  m_codeGen            = m_codeList->get<LatexCodeGenerator>(OutputType::Latex);
   m_codeGen->setTextStream(&m_t);
   m_firstDescItem      = og.m_firstDescItem;
   m_disableLinks       = og.m_disableLinks;
@@ -294,7 +299,7 @@ LatexGenerator &LatexGenerator::operator=(const LatexGenerator &og)
   {
     m_dir                = og.m_dir;
     m_codeList           = std::make_unique<OutputCodeList>(*og.m_codeList);
-    m_codeGen            = m_codeList->get<LatexCodeGenerator>();
+    m_codeGen            = m_codeList->get<LatexCodeGenerator>(OutputType::Latex);
     m_codeGen->setTextStream(&m_t);
     m_firstDescItem      = og.m_firstDescItem;
     m_disableLinks       = og.m_disableLinks;
@@ -306,27 +311,11 @@ LatexGenerator &LatexGenerator::operator=(const LatexGenerator &og)
   return *this;
 }
 
-LatexGenerator::LatexGenerator(LatexGenerator &&og)
-  : OutputGenerator(std::move(og))
-{
-  m_codeList           = std::exchange(og.m_codeList,std::unique_ptr<OutputCodeList>());
-  m_codeGen            = m_codeList->get<LatexCodeGenerator>();
-  m_codeGen->setTextStream(&m_t);
-  m_firstDescItem      = std::exchange(og.m_firstDescItem,true);
-  m_disableLinks       = std::exchange(og.m_disableLinks,false);
-  m_relPath            = std::exchange(og.m_relPath,QCString());
-  m_indent             = std::exchange(og.m_indent,0);
-  m_templateMemberItem = std::exchange(og.m_templateMemberItem,false);
-  m_hierarchyLevel     = og.m_hierarchyLevel;
-}
-
-LatexGenerator::~LatexGenerator()
-{
-}
+LatexGenerator::~LatexGenerator() = default;
 
 void LatexGenerator::addCodeGen(OutputCodeList &list)
 {
-  list.add(OutputCodeList::OutputCodeVariant(LatexCodeGeneratorDefer(m_codeGen)));
+  list.add<LatexCodeGeneratorDefer>(m_codeGen);
 }
 
 static void writeLatexMakefile()
@@ -371,7 +360,11 @@ static void writeLatexMakefile()
     t << "\tps2pdf $(MANUAL_FILE).ps $(MANUAL_FILE).pdf\n\n";
     t << "$(MANUAL_FILE).dvi: clean $(MANUAL_FILE).tex doxygen.sty\n"
       << "\techo \"Running latex...\"\n"
-      << "\t$(LATEX_CMD) $(MANUAL_FILE).tex\n"
+      << "\t$(LATEX_CMD) $(MANUAL_FILE).tex || \\\n"
+      << "\tif [ $$? != 0 ] ; then \\\n"
+      << "\t        \\echo \"Please consult $(MANUAL_FILE).log to see the error messages\" ; \\\n"
+      << "\t        false; \\\n"
+      << "\tfi\n"
       << "\techo \"Running makeindex...\"\n"
       << "\t$(MKIDX_CMD) $(MANUAL_FILE).idx\n";
     if (generateBib)
@@ -379,7 +372,11 @@ static void writeLatexMakefile()
       t << "\techo \"Running bibtex...\"\n";
       t << "\t$(BIBTEX_CMD) $(MANUAL_FILE)\n";
       t << "\techo \"Rerunning latex....\"\n";
-      t << "\t$(LATEX_CMD) $(MANUAL_FILE).tex\n";
+      t << "\t$(LATEX_CMD) $(MANUAL_FILE).tex || \\\n"
+        << "\tif [ $$? != 0 ] ; then \\\n"
+        << "\t        \\echo \"Please consult $(MANUAL_FILE).log to see the error messages\" ; \\\n"
+        << "\t        false; \\\n"
+        << "\tfi\n";
     }
     t << "\techo \"Rerunning latex....\"\n"
       << "\t$(LATEX_CMD) $(MANUAL_FILE).tex\n"
@@ -388,10 +385,19 @@ static void writeLatexMakefile()
       << "\t    do \\\n"
       << "\t      echo \"Rerunning latex....\" ;\\\n"
       << "\t      $(LATEX_CMD) $(MANUAL_FILE).tex ; \\\n"
+      << "\t      $(LATEX_CMD) $(MANUAL_FILE).tex || \\\n"
+      << "\t      if [ $$? != 0 ] ; then \\\n"
+      << "\t              \\echo \"Please consult $(MANUAL_FILE).log to see the error messages\" ; \\\n"
+      << "\t              false; \\\n"
+      << "\t      fi; \\\n"
       << "\t      latex_count=`expr $$latex_count - 1` ;\\\n"
       << "\t    done\n"
       << "\t$(MKIDX_CMD) $(MANUAL_FILE).idx\n"
-      << "\t$(LATEX_CMD) $(MANUAL_FILE).tex\n\n"
+      << "\t$(LATEX_CMD) $(MANUAL_FILE).tex || \\\n"
+      << "\tif [ $$? != 0 ] ; then \\\n"
+      << "\t        \\echo \"Please consult $(MANUAL_FILE).log to see the error messages\" ; \\\n"
+      << "\t        false; \\\n"
+      << "\tfi\n"
       << "$(MANUAL_FILE).ps: $(MANUAL_FILE).ps\n"
       << "\tpsnup -2 $(MANUAL_FILE).ps >$(MANUAL_FILE).ps\n"
       << "\n"
@@ -403,23 +409,43 @@ static void writeLatexMakefile()
     t << "all: $(MANUAL_FILE).pdf\n\n"
       << "pdf: $(MANUAL_FILE).pdf\n\n";
     t << "$(MANUAL_FILE).pdf: clean $(MANUAL_FILE).tex\n";
-    t << "\t$(LATEX_CMD) $(MANUAL_FILE)\n";
+    t << "\t$(LATEX_CMD) $(MANUAL_FILE) || \\\n"
+      << "\tif [ $$? != 0 ] ; then \\\n"
+      << "\t        \\echo \"Please consult $(MANUAL_FILE).log to see the error messages\" ; \\\n"
+      << "\t        false; \\\n"
+      << "\tfi\n";
     t << "\t$(MKIDX_CMD) $(MANUAL_FILE).idx\n";
     if (generateBib)
     {
       t << "\t$(BIBTEX_CMD) $(MANUAL_FILE)\n";
-      t << "\t$(LATEX_CMD) $(MANUAL_FILE)\n";
+      t << "\t$(LATEX_CMD) $(MANUAL_FILE) || \\\n"
+        << "\tif [ $$? != 0 ] ; then \\\n"
+        << "\t        \\echo \"Please consult $(MANUAL_FILE).log to see the error messages\" ; \\\n"
+        << "\t        false; \\\n"
+        << "\tfi\n";
     }
-    t << "\t$(LATEX_CMD) $(MANUAL_FILE)\n"
+    t << "\t$(LATEX_CMD) $(MANUAL_FILE) || \\\n"
+      << "\tif [ $$? != 0 ] ; then \\\n"
+      << "\t        \\echo \"Please consult $(MANUAL_FILE).log to see the error messages\" ; \\\n"
+      << "\t        false; \\\n"
+      << "\tfi\n"
       << "\tlatex_count=$(LATEX_COUNT) ; \\\n"
       << "\twhile grep -E -s 'Rerun (LaTeX|to get cross-references right|to get bibliographical references right)' $(MANUAL_FILE).log && [ $$latex_count -gt 0 ] ;\\\n"
       << "\t    do \\\n"
       << "\t      echo \"Rerunning latex....\" ;\\\n"
-      << "\t      $(LATEX_CMD) $(MANUAL_FILE) ;\\\n"
+      << "\t      $(LATEX_CMD) $(MANUAL_FILE) || \\\n"
+      << "\t      if [ $$? != 0 ] ; then \\\n"
+      << "\t              \\echo \"Please consult $(MANUAL_FILE).log to see the error messages\" ; \\\n"
+      << "\t              false; \\\n"
+      << "\t      fi; \\\n"
       << "\t      latex_count=`expr $$latex_count - 1` ;\\\n"
       << "\t    done\n"
       << "\t$(MKIDX_CMD) $(MANUAL_FILE).idx\n"
-      << "\t$(LATEX_CMD) $(MANUAL_FILE)\n\n";
+      << "\t$(LATEX_CMD) $(MANUAL_FILE) || \\\n"
+      << "\tif [ $$? != 0 ] ; then \\\n"
+      << "\t        \\echo \"Please consult $(MANUAL_FILE).log to see the error messages\" ; \\\n"
+      << "\t        false; \\\n"
+      << "\tfi\n";
   }
 
   t << "\n"
@@ -445,7 +471,7 @@ static void writeMakeBat()
     term("Could not open file %s for writing\n",qPrint(fileName));
   }
   t << "pushd %~dp0\r\n";
-  t << "if not %errorlevel% == 0 goto :end\r\n";
+  t << "if not %errorlevel% == 0 goto :end1\r\n";
   t << "\r\n";
   t << "set ORG_LATEX_CMD=%LATEX_CMD%\r\n";
   t << "set ORG_MKIDX_CMD=%MKIDX_CMD%\r\n";
@@ -463,6 +489,7 @@ static void writeMakeBat()
   if (!Config_getBool(USE_PDFLATEX)) // use plain old latex
   {
     t << "%LATEX_CMD% %MANUAL_FILE%.tex\r\n";
+    t << "@if ERRORLEVEL 1 goto :error\r\n";
     t << "echo ----\r\n";
     t << "%MKIDX_CMD% %MANUAL_FILE%.idx\r\n";
     if (generateBib)
@@ -470,6 +497,7 @@ static void writeMakeBat()
       t << "%BIBTEX_CMD% %MANUAL_FILE%\r\n";
       t << "echo ----\r\n";
       t << "\t%LATEX_CMD% %MANUAL_FILE%.tex\r\n";
+      t << "@if ERRORLEVEL 1 goto :error\r\n";
     }
     t << "setlocal enabledelayedexpansion\r\n";
     t << "set count=%LAT#EX_COUNT%\r\n";
@@ -483,11 +511,13 @@ static void writeMakeBat()
     t << "if !count! EQU 0 goto :skip\r\n\r\n";
     t << "echo ----\r\n";
     t << "%LATEX_CMD% %MANUAL_FILE%.tex\r\n";
+    t << "@if ERRORLEVEL 1 goto :error\r\n";
     t << "goto :repeat\r\n";
     t << ":skip\r\n";
     t << "endlocal\r\n";
     t << "%MKIDX_CMD% %MANUAL_FILE%.idx\r\n";
     t << "%LATEX_CMD% %MANUAL_FILE%.tex\r\n";
+    t << "@if ERRORLEVEL 1 goto :error\r\n";
     t << "dvips -o %MANUAL_FILE%.ps %MANUAL_FILE%.dvi\r\n";
     t << Portable::ghostScriptCommand();
     t << " -q -dNOPAUSE -dBATCH -sDEVICE=pdfwrite "
@@ -496,15 +526,19 @@ static void writeMakeBat()
   else // use pdflatex
   {
     t << "%LATEX_CMD% %MANUAL_FILE%\r\n";
+    t << "@if ERRORLEVEL 1 goto :error\r\n";
     t << "echo ----\r\n";
     t << "%MKIDX_CMD% %MANUAL_FILE%.idx\r\n";
     if (generateBib)
     {
       t << "%BIBTEX_CMD% %MANUAL_FILE%\r\n";
       t << "%LATEX_CMD% %MANUAL_FILE%\r\n";
+      t << "@if ERRORLEVEL 1 goto :error\r\n";
     }
     t << "echo ----\r\n";
-    t << "%LATEX_CMD% %MANUAL_FILE%\r\n\r\n";
+    t << "%LATEX_CMD% %MANUAL_FILE%\r\n";
+    t << "@if ERRORLEVEL 1 goto :error\r\n";
+    t << "\r\n";
     t << "setlocal enabledelayedexpansion\r\n";
     t << "set count=%LATEX_COUNT%\r\n";
     t << ":repeat\r\n";
@@ -517,13 +551,22 @@ static void writeMakeBat()
     t << "if !count! EQU 0 goto :skip\r\n\r\n";
     t << "echo ----\r\n";
     t << "%LATEX_CMD% %MANUAL_FILE%\r\n";
+    t << "@if ERRORLEVEL 1 goto :error\r\n";
     t << "goto :repeat\r\n";
     t << ":skip\r\n";
     t << "endlocal\r\n";
     t << "%MKIDX_CMD% %MANUAL_FILE%.idx\r\n";
     t << "%LATEX_CMD% %MANUAL_FILE%\r\n";
+    t << "@if ERRORLEVEL 1 goto :error\r\n";
   }
   t<< "\r\n";
+  t << "goto :end\r\n";
+  t << ":error\r\n";
+  t << "@echo ===============\r\n";
+  t << "@echo Please consult %MANUAL_FILE%.log to see the error messages\r\n";
+  t << "@echo ===============\r\n";
+  t<< "\r\n";
+  t<< ":end\r\n";
   t<< "@REM reset environment\r\n";
   t<< "popd\r\n";
   t<< "set LATEX_CMD=%ORG_LATEX_CMD%\r\n";
@@ -537,7 +580,7 @@ static void writeMakeBat()
   t<< "set LATEX_COUNT=%ORG_LATEX_COUNT%\r\n";
   t<< "set ORG_LATEX_COUNT=\r\n";
   t<< "\r\n";
-  t<< ":end\r\n";
+  t<< ":end1\r\n";
 #endif
 }
 
@@ -554,19 +597,27 @@ void LatexGenerator::init()
   {
     g_header=fileToString(Config_getString(LATEX_HEADER));
     //printf("g_header='%s'\n",qPrint(g_header));
+    QCString result = substituteLatexKeywords(g_header,QCString());
+    checkBlocks(result,Config_getString(LATEX_HEADER),latexMarkerInfo);
   }
   else
   {
     g_header = ResourceMgr::instance().getAsString("header.tex");
+    QCString result = substituteLatexKeywords(g_header,QCString());
+    checkBlocks(result,"<default header.tex>",latexMarkerInfo);
   }
   if (!Config_getString(LATEX_FOOTER).isEmpty())
   {
     g_footer=fileToString(Config_getString(LATEX_FOOTER));
     //printf("g_footer='%s'\n",qPrint(g_footer));
+    QCString result = substituteLatexKeywords(g_footer,QCString());
+    checkBlocks(result,Config_getString(LATEX_FOOTER),latexMarkerInfo);
   }
   else
   {
     g_footer = ResourceMgr::instance().getAsString("footer.tex");
+    QCString result = substituteLatexKeywords(g_footer,QCString());
+    checkBlocks(result,"<default footer.tex>",latexMarkerInfo);
   }
 
   writeLatexMakefile();
@@ -791,8 +842,6 @@ static QCString substituteLatexKeywords(const QCString &str,
     { "$latex_batchmode",          [&]() { return latex_batchmode();                            } }
   });
 
-  static const SelectionMarkerInfo latexMarkerInfo = { '%', "%%BEGIN ",8 ,"%%END ",6, "",0 };
-
   // remove conditional blocks
   result = selectBlocks(result,
   {
@@ -801,7 +850,7 @@ static QCString substituteLatexKeywords(const QCString &str,
     { "COMPACT_LATEX",     compactLatex                           },
     { "PDF_HYPERLINKS",    pdfHyperlinks                          },
     { "USE_PDFLATEX",      usePdfLatex                            },
-    { "LATEX_TIMESTAMP",   timeStamp!=TIMESTAMP_t::NO             },
+    { "TIMESTAMP",         timeStamp!=TIMESTAMP_t::NO             },
     { "LATEX_FONTENC",     !latexFontenc.isEmpty()                },
     { "FORMULA_MACROFILE", !formulaMacrofile.isEmpty()            },
     { "PROJECT_NUMBER",    !projectNumber.isEmpty()               }
@@ -828,6 +877,10 @@ void LatexGenerator::startIndexSection(IndexSection is)
       if (compactLatex) m_t << "\\doxysection"; else m_t << "\\chapter";
       m_t << "{"; //Module Index}\n"
       break;
+    case IndexSection::isTopicIndex:
+      if (compactLatex) m_t << "\\doxysection"; else m_t << "\\chapter";
+      m_t << "{"; //Topic Index}\n"
+      break;
     case IndexSection::isDirIndex:
       if (compactLatex) m_t << "\\doxysection"; else m_t << "\\chapter";
       m_t << "{"; //Directory Index}\n"
@@ -856,14 +909,27 @@ void LatexGenerator::startIndexSection(IndexSection is)
       if (compactLatex) m_t << "\\doxysection"; else m_t << "\\chapter";
       m_t << "{"; //Annotated Page Index}\n"
       break;
-    case IndexSection::isModuleDocumentation:
+    case IndexSection::isTopicDocumentation:
       {
         for (const auto &gd : *Doxygen::groupLinkedMap)
         {
           if (!gd->isReference())
           {
             if (compactLatex) m_t << "\\doxysection"; else m_t << "\\chapter";
-            m_t << "{"; //Module Documentation}\n";
+            m_t << "{"; //Topic Documentation}\n";
+            break;
+          }
+        }
+      }
+      break;
+    case IndexSection::isModuleDocumentation:
+      {
+        for (const auto &mod : ModuleManager::instance().modules())
+        {
+          if (!mod->isReference() && mod->isPrimaryInterface())
+          {
+            if (compactLatex) m_t << "\\doxysection"; else m_t << "\\chapter";
+            m_t << "{"; //Topic Documentation}\n";
             break;
           }
         }
@@ -876,7 +942,7 @@ void LatexGenerator::startIndexSection(IndexSection is)
           if (dd->isLinkableInProject())
           {
             if (compactLatex) m_t << "\\doxysection"; else m_t << "\\chapter";
-            m_t << "{"; //Module Documentation}\n";
+            m_t << "{"; //Dir Documentation}\n";
             break;
           }
         }
@@ -913,7 +979,7 @@ void LatexGenerator::startIndexSection(IndexSection is)
         for (const auto &cd : *Doxygen::classLinkedMap)
         {
           if (cd->isLinkableInProject() &&
-              cd->templateMaster()==0 &&
+              cd->templateMaster()==nullptr &&
               !cd->isEmbeddedInOuterScope() &&
               !cd->isAlias()
              )
@@ -980,6 +1046,9 @@ void LatexGenerator::endIndexSection(IndexSection is)
     case IndexSection::isModuleIndex:
       m_t << "}\n\\input{modules}\n";
       break;
+    case IndexSection::isTopicIndex:
+      m_t << "}\n\\input{topics}\n";
+      break;
     case IndexSection::isDirIndex:
       m_t << "}\n\\input{dirs}\n";
       break;
@@ -1001,7 +1070,7 @@ void LatexGenerator::endIndexSection(IndexSection is)
     case IndexSection::isPageIndex:
       m_t << "}\n\\input{pages}\n";
       break;
-    case IndexSection::isModuleDocumentation:
+    case IndexSection::isTopicDocumentation:
       {
         m_t << "}\n";
         for (const auto &gd : *Doxygen::groupLinkedMap)
@@ -1013,6 +1082,18 @@ void LatexGenerator::endIndexSection(IndexSection is)
         }
       }
       break;
+    case IndexSection::isModuleDocumentation:
+      {
+        m_t << "}\n";
+        for (const auto &mod : ModuleManager::instance().modules())
+        {
+          if (!mod->isReference() && mod->isPrimaryInterface())
+          {
+            writePageLink(mod->getOutputFileBase(), FALSE);
+          }
+        }
+      }
+      break;
     case IndexSection::isDirDocumentation:
       {
         bool found=FALSE;
@@ -1070,7 +1151,7 @@ void LatexGenerator::endIndexSection(IndexSection is)
         for (const auto &cd : *Doxygen::classLinkedMap)
         {
           if (cd->isLinkableInProject() &&
-              cd->templateMaster()==0 &&
+              cd->templateMaster()==nullptr &&
              !cd->isEmbeddedInOuterScope() &&
              !cd->isAlias()
              )
@@ -1523,6 +1604,14 @@ void LatexGenerator::startDoxyAnchor(const QCString &fName,const QCString &,
     if (!anchor.isEmpty()) m_t << "_" << anchor;
     m_t << "}";
   }
+}
+
+void LatexGenerator::endDoxyAnchor(const QCString &/* fName */,const QCString &/* anchor */)
+{
+}
+
+void LatexGenerator::addLabel(const QCString &fName, const QCString &anchor)
+{
   m_t << "\\label{";
   if (!fName.isEmpty()) m_t << stripPath(fName);
   if (!anchor.isEmpty()) m_t << "_" << anchor;
@@ -1530,10 +1619,6 @@ void LatexGenerator::startDoxyAnchor(const QCString &fName,const QCString &,
   m_t << "} \n";
 }
 
-void LatexGenerator::endDoxyAnchor(const QCString &/* fName */,const QCString &/* anchor */)
-{
-}
-
 void LatexGenerator::writeAnchor(const QCString &fName,const QCString &name)
 {
   //printf("LatexGenerator::writeAnchor(%s,%s)\n",fName,name);
@@ -1592,26 +1677,30 @@ void LatexGenerator::startSection(const QCString &lab,const QCString &,SectionTy
   m_t << "\\";
   if (Config_getBool(COMPACT_LATEX))
   {
-    switch(type)
+    switch(type.level())
     {
-      case SectionType::Page:          m_t << "doxysubsection"; break;
-      case SectionType::Section:       m_t << "doxysubsubsection"; break;
-      case SectionType::Subsection:    m_t << "doxyparagraph"; break;
-      case SectionType::Subsubsection: m_t << "doxysubparagraph"; break;
-      case SectionType::Paragraph:     m_t << "doxysubparagraph"; break;
+      case SectionType::Page:             m_t << "doxysubsection"; break;
+      case SectionType::Section:          m_t << "doxysubsubsection"; break;
+      case SectionType::Subsection:       m_t << "doxysubsubsubsection"; break;
+      case SectionType::Subsubsection:    m_t << "doxysubsubsubsubsection"; break;
+      case SectionType::Paragraph:        m_t << "doxysubsubsubsubsubsection"; break;
+      case SectionType::Subparagraph:     m_t << "doxysubsubsubsubsubsubsection"; break;
+      case SectionType::Subsubparagraph:  m_t << "doxysubsubsubsubsubsubsection"; break;
       default: ASSERT(0); break;
     }
     m_t << "{";
   }
   else
   {
-    switch(type)
+    switch(type.level())
     {
-      case SectionType::Page:          m_t << "doxysection"; break;
-      case SectionType::Section:       m_t << "doxysubsection"; break;
-      case SectionType::Subsection:    m_t << "doxysubsubsection"; break;
-      case SectionType::Subsubsection: m_t << "doxyparagraph"; break;
-      case SectionType::Paragraph:     m_t << "doxysubparagraph"; break;
+      case SectionType::Page:             m_t << "doxysection"; break;
+      case SectionType::Section:          m_t << "doxysubsection"; break;
+      case SectionType::Subsection:       m_t << "doxysubsubsection"; break;
+      case SectionType::Subsubsection:    m_t << "doxysubsubsubsection"; break;
+      case SectionType::Paragraph:        m_t << "doxysubsubsubsubsection"; break;
+      case SectionType::Subparagraph:     m_t << "doxysubsubsubsubsubsection"; break;
+      case SectionType::Subsubparagraph:  m_t << "doxysubsubsubsubsubsubsection"; break;
       default: ASSERT(0); break;
     }
     m_t << "{";
@@ -1767,10 +1856,10 @@ void LatexGenerator::writeNonBreakableSpace(int)
 // - endDescTableRow()
 // endDescTable()
 
-void LatexGenerator::startDescTable(const QCString &title)
+void LatexGenerator::startDescTable(const QCString &title,const bool hasInits)
 {
   m_codeGen->incUsedTableLevel();
-  m_t << "\\begin{DoxyEnumFields}{" << title << "}\n";
+  m_t << "\\begin{DoxyEnumFields}[" << (hasInits?3:2) << "]{" << title << "}\n";
 }
 
 void LatexGenerator::endDescTable()
@@ -1799,6 +1888,15 @@ void LatexGenerator::endDescTableTitle()
 {
 }
 
+void LatexGenerator::startDescTableInit()
+{
+  m_t << "&";
+}
+
+void LatexGenerator::endDescTableInit()
+{
+}
+
 void LatexGenerator::startDescTableData()
 {
   m_t << "&";
@@ -1881,7 +1979,7 @@ void LatexGenerator::startDotGraph()
 
 void LatexGenerator::endDotGraph(DotClassGraph &g)
 {
-  g.writeGraph(m_t,GOF_EPS,EOF_LaTeX,dir(),fileName(),m_relPath);
+  g.writeGraph(m_t,GraphOutputFormat::EPS,EmbeddedOutputFormat::LaTeX,dir(),fileName(),m_relPath);
 }
 
 void LatexGenerator::startInclDepGraph()
@@ -1890,7 +1988,7 @@ void LatexGenerator::startInclDepGraph()
 
 void LatexGenerator::endInclDepGraph(DotInclDepGraph &g)
 {
-  g.writeGraph(m_t,GOF_EPS,EOF_LaTeX,dir(),fileName(),m_relPath);
+  g.writeGraph(m_t,GraphOutputFormat::EPS,EmbeddedOutputFormat::LaTeX,dir(),fileName(),m_relPath);
 }
 
 void LatexGenerator::startGroupCollaboration()
@@ -1899,7 +1997,7 @@ void LatexGenerator::startGroupCollaboration()
 
 void LatexGenerator::endGroupCollaboration(DotGroupCollaboration &g)
 {
-  g.writeGraph(m_t,GOF_EPS,EOF_LaTeX,dir(),fileName(),m_relPath);
+  g.writeGraph(m_t,GraphOutputFormat::EPS,EmbeddedOutputFormat::LaTeX,dir(),fileName(),m_relPath);
 }
 
 void LatexGenerator::startCallGraph()
@@ -1908,7 +2006,7 @@ void LatexGenerator::startCallGraph()
 
 void LatexGenerator::endCallGraph(DotCallGraph &g)
 {
-  g.writeGraph(m_t,GOF_EPS,EOF_LaTeX,dir(),fileName(),m_relPath);
+  g.writeGraph(m_t,GraphOutputFormat::EPS,EmbeddedOutputFormat::LaTeX,dir(),fileName(),m_relPath);
 }
 
 void LatexGenerator::startDirDepGraph()
@@ -1917,7 +2015,7 @@ void LatexGenerator::startDirDepGraph()
 
 void LatexGenerator::endDirDepGraph(DotDirDeps &g)
 {
-  g.writeGraph(m_t,GOF_EPS,EOF_LaTeX,dir(),fileName(),m_relPath);
+  g.writeGraph(m_t,GraphOutputFormat::EPS,EmbeddedOutputFormat::LaTeX,dir(),fileName(),m_relPath);
 }
 
 void LatexGenerator::startExamples()
@@ -1959,9 +2057,19 @@ void LatexGenerator::startParameterName(bool /*oneArgOnly*/)
   m_t << "{";
 }
 
-void LatexGenerator::endParameterName(bool last,bool /*emptyList*/,bool closeBracket)
+void LatexGenerator::endParameterName()
 {
-  m_t << " }";
+  m_t << "}";
+}
+
+void LatexGenerator::startParameterExtra()
+{
+  m_t << "{";
+}
+
+void LatexGenerator::endParameterExtra(bool last,bool /*emptyList*/,bool closeBracket)
+{
+  m_t << "}";
   if (last)
   {
     m_t << "\\end{DoxyParamCaption}";
@@ -2205,6 +2313,7 @@ void writeLatexSpecialFormulaChars(TextStream &t)
     sup3[1]= 0xB3;
     sup3[2]= 0;
 
+    t << "\\ifPDFTeX\n";
     t << "\\usepackage{newunicodechar}\n";
     // taken from the newunicodechar package and removed the warning message
     // actually forcing to redefine the unicode character
@@ -2228,6 +2337,7 @@ void writeLatexSpecialFormulaChars(TextStream &t)
          "  \\doxynewunicodechar{" << sup2  << "}{${}^{2}$}% Superscript two\n"
          "  \\doxynewunicodechar{" << sup3  << "}{${}^{3}$}% Superscript three\n"
          "\n";
+    t << "\\fi\n";
 }
 
 void filterLatexString(TextStream &t,const QCString &str,
@@ -2236,15 +2346,13 @@ void filterLatexString(TextStream &t,const QCString &str,
   if (str.isEmpty()) return;
   bool pdfHyperlinks = Config_getBool(PDF_HYPERLINKS);
   //printf("filterLatexString(%s) insideTabbing=%d\n",qPrint(str),insideTabbing);
-  const char *p=str.data();
-  const char *q;
-  int cnt;
-  unsigned char c;
+  const char *p = str.data();
+  const char *q = nullptr;
   unsigned char pc='\0';
 
   while (*p)
   {
-    c=static_cast<unsigned char>(*p++);
+    unsigned char c=static_cast<unsigned char>(*p++);
 
     if (insidePre)
     {
@@ -2300,33 +2408,35 @@ void filterLatexString(TextStream &t,const QCString &str,
         case '$':  t << "\\$";           break;
         case '%':  t << "\\%";           break;
         case '^':  processEntity(t,pdfHyperlinks,"$^\\wedge$","\\string^");    break;
-        case '&':  // possibility to have a special symbol
-                   q = p;
-                   cnt = 2; // we have to count & and ; as well
-                   while ((*q >= 'a' && *q <= 'z') || (*q >= 'A' && *q <= 'Z') || (*q >= '0' && *q <= '9'))
-                   {
-                     cnt++;
-                     q++;
-                   }
-                   if (*q == ';')
-                   {
-                      --p; // we need & as well
-                      HtmlEntityMapper::SymType res = HtmlEntityMapper::instance().name2sym(QCString(p).left(cnt));
-                      if (res == HtmlEntityMapper::Sym_Unknown)
-                      {
-                        p++;
-                        t << "\\&";
-                      }
-                      else
-                      {
-                        t << HtmlEntityMapper::instance().latex(res);
-                        q++;
-                        p = q;
-                      }
-                   }
-                   else
-                   {
-                     t << "\\&";
+        case '&':  {
+                     // possibility to have a special symbol
+                     q = p;
+                     int cnt = 2; // we have to count & and ; as well
+                     while ((*q >= 'a' && *q <= 'z') || (*q >= 'A' && *q <= 'Z') || (*q >= '0' && *q <= '9'))
+                     {
+                       cnt++;
+                       q++;
+                     }
+                     if (*q == ';')
+                     {
+                       --p; // we need & as well
+                       HtmlEntityMapper::SymType res = HtmlEntityMapper::instance().name2sym(QCString(p).left(cnt));
+                       if (res == HtmlEntityMapper::Sym_Unknown)
+                       {
+                         p++;
+                         t << "\\&";
+                       }
+                       else
+                       {
+                         t << HtmlEntityMapper::instance().latex(res);
+                         q++;
+                         p = q;
+                       }
+                     }
+                     else
+                     {
+                       t << "\\&";
+                     }
                    }
                    break;
         case '*':  processEntity(t,pdfHyperlinks,"$\\ast$","*");    break;
@@ -2355,7 +2465,7 @@ void filterLatexString(TextStream &t,const QCString &str,
                    break;
         case '\\': t << "\\textbackslash{}";
                    break;
-        case '"':  t << "\\\"{}";
+        case '"':  t << "\"{}";
                    break;
         case '`':  t << "\\`{}";
                    break;
@@ -2399,11 +2509,10 @@ QCString latexEscapeLabelName(const QCString &s)
 {
   //printf("latexEscapeLabelName(%s)\n",qPrint(s));
   if (s.isEmpty()) return s;
-  QCString tmp(s.length()+1);
+  QCString tmp(s.length(), QCString::ExplicitSize);
   TextStream t;
   const char *p=s.data();
-  char c;
-  int i;
+  char c = 0;
   while ((c=*p++))
   {
     switch (c)
@@ -2417,22 +2526,24 @@ QCString latexEscapeLabelName(const QCString &s)
       case '~': t << "````~"; break; // to get it a bit better in index together with other special characters
       // NOTE: adding a case here, means adding it to while below as well!
       default:
-        i=0;
-        // collect as long string as possible, before handing it to docify
-        tmp[i++]=c;
-        while ((c=*p) && c!='@' && c!='[' && c!=']' && c!='!' && c!='{' && c!='}' && c!='|')
         {
+          int i=0;
+          // collect as long string as possible, before handing it to docify
           tmp[i++]=c;
-          p++;
+          while ((c=*p) && c!='@' && c!='[' && c!=']' && c!='!' && c!='{' && c!='}' && c!='|')
+          {
+            tmp[i++]=c;
+            p++;
+          }
+          tmp[i]=0;
+          filterLatexString(t,tmp,
+                            true,  // insideTabbing
+                            false, // insidePre
+                            false, // insideItem
+                            false, // insideTable
+                            false  // keepSpaces
+                           );
         }
-        tmp[i]=0;
-        filterLatexString(t,tmp,
-                          true,  // insideTabbing
-                          false, // insidePre
-                          false, // insideItem
-                          false, // insideTable
-                          false  // keepSpaces
-                         );
         break;
     }
   }
@@ -2443,11 +2554,10 @@ QCString latexEscapeIndexChars(const QCString &s)
 {
   //printf("latexEscapeIndexChars(%s)\n",qPrint(s));
   if (s.isEmpty()) return s;
-  QCString tmp(s.length()+1);
+  QCString tmp(s.length(), QCString::ExplicitSize);
   TextStream t;
   const char *p=s.data();
-  char c;
-  int i;
+  char c = 0;
   while ((c=*p++))
   {
     switch (c)
@@ -2462,22 +2572,24 @@ QCString latexEscapeIndexChars(const QCString &s)
       case '}': t << "\\rcurly{}"; break;
       // NOTE: adding a case here, means adding it to while below as well!
       default:
-        i=0;
-        // collect as long string as possible, before handing it to docify
-        tmp[i++]=c;
-        while ((c=*p) && c!='"' && c!='@' && c!='[' && c!=']' && c!='!' && c!='{' && c!='}' && c!='|')
         {
+          int i=0;
+          // collect as long string as possible, before handing it to docify
           tmp[i++]=c;
-          p++;
+          while ((c=*p) && c!='"' && c!='@' && c!='[' && c!=']' && c!='!' && c!='{' && c!='}' && c!='|')
+          {
+            tmp[i++]=c;
+            p++;
+          }
+          tmp[i]=0;
+          filterLatexString(t,tmp,
+                            true,   // insideTabbing
+                            false,  // insidePre
+                            false,  // insideItem
+                            false,  // insideTable
+                            false   // keepSpaces
+                           );
         }
-        tmp[i]=0;
-        filterLatexString(t,tmp,
-                          true,   // insideTabbing
-                          false,  // insidePre
-                          false,  // insideItem
-                          false,  // insideTable
-                          false   // keepSpaces
-                         );
         break;
     }
   }
@@ -2489,7 +2601,7 @@ QCString latexEscapePDFString(const QCString &s)
   if (s.isEmpty()) return s;
   TextStream t;
   const char *p=s.data();
-  char c;
+  char c = 0;
   while ((c=*p++))
   {
     switch (c)
@@ -2518,7 +2630,7 @@ QCString latexFilterURL(const QCString &s)
   if (s.isEmpty()) return s;
   TextStream t;
   const char *p=s.data();
-  char c;
+  char c = 0;
   while ((c=*p++))
   {
     switch (c)
