@@ -55,6 +55,17 @@ static QCString align(const DocHtmlCell &cell)
   return "";
 }
 
+static QCString makeBaseName(const QCString &name)
+{
+  QCString baseName = name;
+  int i = baseName.findRev('/');
+  if (i!=-1)
+  {
+    baseName=baseName.mid(i+1);
+  }
+  return baseName;
+}
+
 RTFDocVisitor::RTFDocVisitor(TextStream &t,OutputCodeList &ci,
                              const QCString &langExt, int hierarchyLevel)
   : m_t(t), m_ci(ci), m_langExt(langExt), m_hierarchyLevel(hierarchyLevel)
@@ -68,6 +79,19 @@ QCString RTFDocVisitor::getStyle(const QCString &name)
   return sd.reference();
 }
 
+QCString RTFDocVisitor::getListTable(const int id)
+{
+  for (int i=0 ; rtf_Table_Default[i].definition ; i++ )
+  {
+    if ((id == rtf_Table_Default[i].id) && (m_indentLevel == rtf_Table_Default[i].lvl))
+    {
+      return rtf_Table_Default[i].place;
+    }
+  }
+  ASSERT(0);
+  return "";
+}
+
 int RTFDocVisitor::indentLevel() const
 {
   return std::min(m_indentLevel,maxIndentLevels-1);
@@ -341,7 +365,7 @@ void RTFDocVisitor::operator()(const DocVerbatim &s)
     case DocVerbatim::Dot:
       {
         static int dotindex = 1;
-        QCString fileName(4096);
+        QCString fileName(4096, QCString::ExplicitSize);
 
         fileName.sprintf("%s%d%s",
             qPrint(Config_getString(RTF_OUTPUT)+"/inline_dotgraph_"),
@@ -370,7 +394,7 @@ void RTFDocVisitor::operator()(const DocVerbatim &s)
     case DocVerbatim::Msc:
       {
         static int mscindex = 1;
-        QCString baseName(4096);
+        QCString baseName(4096, QCString::ExplicitSize);
 
         baseName.sprintf("%s%d%s",
             qPrint(Config_getString(RTF_OUTPUT)+"/inline_mscgraph_"),
@@ -456,7 +480,7 @@ void RTFDocVisitor::operator()(const DocInclude &inc)
                                            -1,    // start line
                                            -1,    // end line
                                            FALSE, // inline fragment
-                                           0,     // memberDef
+                                           nullptr,     // memberDef
                                            TRUE   // show line numbers
 					   );
          m_t << "\\par";
@@ -470,11 +494,11 @@ void RTFDocVisitor::operator()(const DocInclude &inc)
       getCodeParser(inc.extension()).parseCode(m_ci,inc.context(),
                                         inc.text(),langExt,inc.isExample(),
                                         inc.exampleFile(),
-                                        0,     // fileDef
+                                        nullptr,     // fileDef
                                         -1,    // startLine
                                         -1,    // endLine
                                         TRUE,  // inlineFragment
-                                        0,     // memberDef
+                                        nullptr,     // memberDef
                                         FALSE  // show line numbers
 				       );
       m_t << "\\par";
@@ -542,7 +566,7 @@ void RTFDocVisitor::operator()(const DocIncOperator &op)
     m_hide = popHidden();
     if (!m_hide)
     {
-      std::unique_ptr<FileDef> fd = 0;
+      std::unique_ptr<FileDef> fd = nullptr;
       if (!op.includeFileName().isEmpty())
       {
         FileInfo cfi( op.includeFileName().str() );
@@ -555,7 +579,7 @@ void RTFDocVisitor::operator()(const DocIncOperator &op)
                                         op.line(),    // startLine
                                         -1,    // endLine
                                         FALSE, // inline fragment
-                                        0,     // memberDef
+                                        nullptr,     // memberDef
                                         op.showLineNo()  // show line numbers
                                        );
     }
@@ -647,6 +671,7 @@ void RTFDocVisitor::operator()(const DocAutoList &l)
   m_t << "{\n";
   int level = indentLevel();
   m_listItemInfo[level].isEnum = l.isEnumList();
+  m_listItemInfo[level].isCheck = l.isCheckedList();
   m_listItemInfo[level].type   = '1';
   m_listItemInfo[level].number = 1;
   m_lastIsPara=FALSE;
@@ -654,16 +679,21 @@ void RTFDocVisitor::operator()(const DocAutoList &l)
   if (!m_lastIsPara) m_t << "\\par";
   m_t << "}\n";
   m_lastIsPara=TRUE;
-  if (indentLevel()==0) m_t << "\\par\n";
+  if (!l.isCheckedList() && indentLevel()==0) m_t << "\\par\n";
 }
 
 void RTFDocVisitor::operator()(const DocAutoListItem &li)
 {
+  static int prevLevel = -1;
   if (m_hide) return;
   DBG_RTF("{\\comment RTFDocVisitor::operator()(const DocAutoListItem &)}\n");
-  if (!m_lastIsPara) m_t << "\\par\n";
-  m_t << rtf_Style_Reset;
   int level = indentLevel();
+  if ((level != prevLevel-1) &&
+      (!(level==prevLevel && level != 0 && m_listItemInfo[level].isCheck)) &&
+      (!m_lastIsPara))
+    m_t << "\\par\n";
+  prevLevel= level;
+  m_t << rtf_Style_Reset;
   if (m_listItemInfo[level].isEnum)
   {
     m_t << getStyle("ListEnum") << "\n";
@@ -672,10 +702,22 @@ void RTFDocVisitor::operator()(const DocAutoListItem &li)
   }
   else
   {
-    m_t << getStyle("ListBullet") << "\n";
+    switch (li.itemNumber())
+    {
+      case DocAutoList::Unchecked: // unchecked
+        m_t << getListTable(2) << "\n";
+        break;
+      case DocAutoList::Checked_x: // checked with x
+      case DocAutoList::Checked_X: // checked with X
+        m_t << getListTable(3) << "\n";
+        break;
+      default:
+        m_t << getListTable(1) << "\n";
+        break;
+    }
   }
   incIndentLevel();
-  m_lastIsPara=FALSE;
+  m_lastIsPara=false;
   visitChildren(li);
   decIndentLevel();
 }
@@ -750,6 +792,8 @@ void RTFDocVisitor::operator()(const DocSimpleSect &s)
       m_t << theTranslator->trRemarks(); break;
     case DocSimpleSect::Attention:
       m_t << theTranslator->trAttention(); break;
+    case DocSimpleSect::Important:
+      m_t << theTranslator->trImportant(); break;
     case DocSimpleSect::User: break;
     case DocSimpleSect::Rcs: break;
     case DocSimpleSect::Unknown:  break;
@@ -797,7 +841,8 @@ void RTFDocVisitor::operator()(const DocSimpleList &l)
   if (m_hide) return;
   DBG_RTF("{\\comment RTFDocVisitor::operator()(const DocSimpleSect &)}\n");
   m_t << "{\n";
-  m_listItemInfo[indentLevel()].isEnum = FALSE;
+  m_listItemInfo[indentLevel()].isEnum = false;
+  m_listItemInfo[indentLevel()].isCheck = false;
   m_lastIsPara=FALSE;
   visitChildren(l);
   if (!m_lastIsPara) m_t << "\\par\n";
@@ -861,6 +906,7 @@ void RTFDocVisitor::operator()(const DocHtmlList &l)
   m_t << "{\n";
   int level = indentLevel();
   m_listItemInfo[level].isEnum = l.type()==DocHtmlList::Ordered;
+  m_listItemInfo[level].isCheck = false;
   m_listItemInfo[level].number = 1;
   m_listItemInfo[level].type   = '1';
   for (const auto &opt : l.attribs())
@@ -871,7 +917,7 @@ void RTFDocVisitor::operator()(const DocHtmlList &l)
     }
     if (opt.name=="start")
     {
-      bool ok;
+      bool ok = false;
       int val = opt.value.toInt(&ok);
       if (ok) m_listItemInfo[level].number = val;
     }
@@ -895,7 +941,7 @@ void RTFDocVisitor::operator()(const DocHtmlListItem &l)
     {
       if (opt.name=="value")
       {
-        bool ok;
+        bool ok = false;
         int val = opt.value.toInt(&ok);
         if (ok) m_listItemInfo[level].number = val;
       }
@@ -1011,7 +1057,7 @@ void RTFDocVisitor::operator()(const DocHtmlRow &r)
 {
   if (m_hide) return;
   DBG_RTF("{\\comment RTFDocVisitor::operator()(const DocHtmlRow &)}\n");
-  size_t i,columnWidth=r.numCells()>0 ? rtf_pageWidth/r.numCells() : 10;
+  size_t columnWidth=r.numCells()>0 ? rtf_pageWidth/r.numCells() : 10;
   m_t << "\\trowd \\trgaph108\\trleft-108"
          "\\trbrdrt\\brdrs\\brdrw10 "
          "\\trbrdrl\\brdrs\\brdrw10 "
@@ -1019,7 +1065,7 @@ void RTFDocVisitor::operator()(const DocHtmlRow &r)
          "\\trbrdrr\\brdrs\\brdrw10 "
          "\\trbrdrh\\brdrs\\brdrw10 "
          "\\trbrdrv\\brdrs\\brdrw10 \n";
-  for (i=0;i<r.numCells();i++)
+  for (size_t i=0;i<r.numCells();i++)
   {
     if (r.isHeading())
     {
@@ -1145,9 +1191,7 @@ void RTFDocVisitor::operator()(const DocHtmlHeader &header)
   m_t << "{" // start section
       << rtf_Style_Reset;
   QCString heading;
-  int level = std::min(header.level()+m_hierarchyLevel,5);
-  if (level <= 0)
-    level = 1;
+  int level = std::clamp(header.level()+m_hierarchyLevel,SectionType::MinLevel,SectionType::MaxLevel);
   heading.sprintf("Heading%d",level);
   // set style
   m_t << rtf_Style[heading.str()].reference();
@@ -1386,7 +1430,6 @@ void RTFDocVisitor::operator()(const DocParamList &pl)
   if (sect && sect->hasTypeSpecifier())  config+=2;
   if (useTable)
   {
-    int i;
     m_t << "\\trowd \\trgaph108\\trleft426\\tblind426"
          "\\trbrdrt\\brdrs\\brdrw10\\brdrcf15 "
          "\\trbrdrl\\brdrs\\brdrw10\\brdrcf15 "
@@ -1394,7 +1437,7 @@ void RTFDocVisitor::operator()(const DocParamList &pl)
          "\\trbrdrr\\brdrs\\brdrw10\\brdrcf15 "
          "\\trbrdrh\\brdrs\\brdrw10\\brdrcf15 "
          "\\trbrdrv\\brdrs\\brdrw10\\brdrcf15 "<< "\n";
-    for (i=0;i<columnPos[config][0];i++)
+    for (int i=0;i<columnPos[config][0];i++)
     {
       m_t << "\\clvertalt\\clbrdrt\\brdrs\\brdrw10\\brdrcf15 "
            "\\clbrdrl\\brdrs\\brdrw10\\brdrcf15 "
@@ -1608,10 +1651,9 @@ void RTFDocVisitor::filter(const QCString &str,bool verbatim)
   if (!str.isEmpty())
   {
     const char *p=str.data();
-    char c;
     while (*p)
     {
-      c=*p++;
+      char c=*p++;
       switch (c)
       {
         case '{':  m_t << "\\{";            break;
@@ -1682,14 +1724,9 @@ void RTFDocVisitor::writeDotFile(const DocDotFile &df)
 void RTFDocVisitor::writeDotFile(const QCString &filename, bool hasCaption,
                                  const QCString &srcFile, int srcLine)
 {
-  QCString baseName=filename;
-  int i;
-  if ((i=baseName.findRev('/'))!=-1)
-  {
-    baseName=baseName.right(baseName.length()-i-1);
-  }
+  QCString baseName=makeBaseName(filename);
   QCString outDir = Config_getString(RTF_OUTPUT);
-  writeDotGraphFromFile(filename,outDir,baseName,GOF_BITMAP,srcFile,srcLine);
+  writeDotGraphFromFile(filename,outDir,baseName,GraphOutputFormat::BITMAP,srcFile,srcLine);
   QCString imgExt = getDotImageExtension();
   includePicturePreRTF(baseName + "." + imgExt, true, hasCaption);
 }
@@ -1701,38 +1738,23 @@ void RTFDocVisitor::writeMscFile(const DocMscFile &df)
 void RTFDocVisitor::writeMscFile(const QCString &fileName, bool hasCaption,
                                  const QCString &srcFile, int srcLine)
 {
-  QCString baseName=fileName;
-  int i;
-  if ((i=baseName.findRev('/'))!=-1)
-  {
-    baseName=baseName.right(baseName.length()-i-1);
-  }
+  QCString baseName=makeBaseName(fileName);
   QCString outDir = Config_getString(RTF_OUTPUT);
-  writeMscGraphFromFile(fileName,outDir,baseName,MSC_BITMAP,srcFile,srcLine);
+  writeMscGraphFromFile(fileName,outDir,baseName,MscOutputFormat::BITMAP,srcFile,srcLine);
   includePicturePreRTF(baseName + ".png", true, hasCaption);
 }
 
 void RTFDocVisitor::writeDiaFile(const DocDiaFile &df)
 {
-  QCString baseName=df.file();
-  int i;
-  if ((i=baseName.findRev('/'))!=-1)
-  {
-    baseName=baseName.right(baseName.length()-i-1);
-  }
+  QCString baseName=makeBaseName(df.file());
   QCString outDir = Config_getString(RTF_OUTPUT);
-  writeDiaGraphFromFile(df.file(),outDir,baseName,DIA_BITMAP,df.srcFile(),df.srcLine());
+  writeDiaGraphFromFile(df.file(),outDir,baseName,DiaOutputFormat::BITMAP,df.srcFile(),df.srcLine());
   includePicturePreRTF(baseName + ".png", true, df.hasCaption());
 }
 
 void RTFDocVisitor::writePlantUMLFile(const QCString &fileName, bool hasCaption)
 {
-  QCString baseName=fileName;
-  int i;
-  if ((i=baseName.findRev('/'))!=-1)
-  {
-    baseName=baseName.right(baseName.length()-i-1);
-  }
+  QCString baseName=makeBaseName(fileName);
   QCString outDir = Config_getString(RTF_OUTPUT);
   PlantumlManager::instance().generatePlantUMLOutput(fileName,outDir,PlantumlManager::PUML_BITMAP);
   includePicturePreRTF(baseName + ".png", true, hasCaption);
