@@ -20,7 +20,6 @@
 #include "settings.h"
 #include "message.h"
 
-#if USE_SQLITE3
 
 #include "sqlite3gen.h"
 #include "doxygen.h"
@@ -49,6 +48,7 @@
 #include "fileinfo.h"
 #include "dir.h"
 #include "datetime.h"
+#include "moduledef.h"
 
 #include <sys/stat.h>
 #include <string.h>
@@ -506,9 +506,9 @@ const char * table_schema[][2] = {
 
 //////////////////////////////////////////////////////
 struct SqlStmt {
-  const char   *query = 0;
-  sqlite3_stmt *stmt = 0;
-  sqlite3 *db = 0;
+  const char   *query = nullptr;
+  sqlite3_stmt *stmt = nullptr;
+  sqlite3 *db = nullptr;
 };
 //////////////////////////////////////////////////////
 /* If you add a new statement below, make sure to add it to
@@ -520,7 +520,7 @@ SqlStmt meta_insert = {
     "( doxygen_version, schema_version, generated_at, generated_on, project_name, project_number, project_brief )"
   "VALUES "
     "(:doxygen_version,:schema_version,:generated_at,:generated_on,:project_name,:project_number,:project_brief )"
-  ,NULL
+  ,nullptr
 };
 //////////////////////////////////////////////////////
 SqlStmt incl_insert = {
@@ -528,12 +528,12 @@ SqlStmt incl_insert = {
     "( local, src_id, dst_id ) "
   "VALUES "
     "(:local,:src_id,:dst_id )"
-  ,NULL
+  ,nullptr
 };
 SqlStmt incl_select = {
   "SELECT COUNT(*) FROM includes WHERE "
   "local=:local AND src_id=:src_id AND dst_id=:dst_id"
-  ,NULL
+  ,nullptr
 };
 //////////////////////////////////////////////////////
 SqlStmt contains_insert={
@@ -541,31 +541,31 @@ SqlStmt contains_insert={
     "( inner_rowid, outer_rowid )"
   "VALUES "
     "(:inner_rowid,:outer_rowid )"
-  ,NULL
+  ,nullptr
 };
 //////////////////////////////////////////////////////
 SqlStmt path_select = {
   "SELECT rowid FROM path WHERE name=:name"
-  ,NULL
+  ,nullptr
 };
 SqlStmt path_insert = {
   "INSERT INTO path "
     "( type, local, found, name )"
   "VALUES "
     "(:type,:local,:found,:name )"
-  ,NULL
+  ,nullptr
 };
 //////////////////////////////////////////////////////
 SqlStmt refid_select =  {
   "SELECT rowid FROM refid WHERE refid=:refid"
-  ,NULL
+  ,nullptr
 };
 SqlStmt refid_insert = {
   "INSERT INTO refid "
     "( refid )"
   "VALUES "
     "(:refid )"
-  ,NULL
+  ,nullptr
 };
 //////////////////////////////////////////////////////
 SqlStmt xrefs_insert= {
@@ -573,19 +573,19 @@ SqlStmt xrefs_insert= {
     "( src_rowid, dst_rowid, context )"
   "VALUES "
     "(:src_rowid,:dst_rowid,:context )"
-  ,NULL
+  ,nullptr
 };//////////////////////////////////////////////////////
 SqlStmt reimplements_insert= {
   "INSERT INTO reimplements "
     "( memberdef_rowid, reimplemented_rowid )"
   "VALUES "
     "(:memberdef_rowid,:reimplemented_rowid )"
-  ,NULL
+  ,nullptr
 };
 //////////////////////////////////////////////////////
 SqlStmt memberdef_exists={
   "SELECT EXISTS (SELECT * FROM memberdef WHERE rowid = :rowid)"
-  ,NULL
+  ,nullptr
 };
 
 SqlStmt memberdef_incomplete={
@@ -593,7 +593,7 @@ SqlStmt memberdef_incomplete={
     "SELECT * FROM memberdef WHERE "
     "rowid = :rowid AND inline != 2 AND inline != :new_inline"
   ")"
-  ,NULL
+  ,nullptr
 };
 
 SqlStmt memberdef_insert={
@@ -715,7 +715,7 @@ SqlStmt memberdef_insert={
     ":briefdescription,"
     ":inbodydescription"
   ")"
-  ,NULL
+  ,nullptr
 };
 /*
 We have a slightly different need than the XML here. The XML can have two
@@ -734,7 +734,7 @@ SqlStmt memberdef_update_decl={
     "briefdescription = 'Declaration: ' || :briefdescription || 'Definition: ' || briefdescription,"
     "inbodydescription = 'Declaration: ' || :inbodydescription || 'Definition: ' || inbodydescription "
   "WHERE rowid = :rowid"
-  ,NULL
+  ,nullptr
 };
 SqlStmt memberdef_update_def={
   "UPDATE memberdef SET "
@@ -746,7 +746,7 @@ SqlStmt memberdef_update_def={
     "briefdescription = 'Declaration: ' || briefdescription || 'Definition: ' || :briefdescription,"
     "inbodydescription = 'Declaration: ' || inbodydescription || 'Definition: ' || :inbodydescription "
   "WHERE rowid = :rowid"
-  ,NULL
+  ,nullptr
 };
 //////////////////////////////////////////////////////
 SqlStmt member_insert={
@@ -754,7 +754,7 @@ SqlStmt member_insert={
     "( scope_rowid, memberdef_rowid, prot, virt ) "
   "VALUES "
     "(:scope_rowid,:memberdef_rowid,:prot,:virt )"
-  ,NULL
+  ,nullptr
 };
 //////////////////////////////////////////////////////
 SqlStmt compounddef_insert={
@@ -786,13 +786,13 @@ SqlStmt compounddef_insert={
     ":briefdescription,"
     ":detaileddescription"
   ")"
-  ,NULL
+  ,nullptr
 };
 SqlStmt compounddef_exists={
   "SELECT EXISTS ("
     "SELECT * FROM compounddef WHERE rowid = :rowid"
   ")"
-  ,NULL
+  ,nullptr
 };
 //////////////////////////////////////////////////////
 SqlStmt compoundref_insert={
@@ -800,7 +800,7 @@ SqlStmt compoundref_insert={
     "( base_rowid, derived_rowid, prot, virt ) "
   "VALUES "
     "(:base_rowid,:derived_rowid,:prot,:virt )"
-  ,NULL
+  ,nullptr
 };
 //////////////////////////////////////////////////////
 SqlStmt param_select = {
@@ -812,14 +812,14 @@ SqlStmt param_select = {
     "(array IS NULL OR array=:array) AND "
     "(defval IS NULL OR defval=:defval) AND "
     "(briefdescription IS NULL OR briefdescription=:briefdescription)"
-  ,NULL
+  ,nullptr
 };
 SqlStmt param_insert = {
   "INSERT INTO param "
     "( attributes, type, declname, defname, array, defval, briefdescription ) "
   "VALUES "
     "(:attributes,:type,:declname,:defname,:array,:defval,:briefdescription)"
-  ,NULL
+  ,nullptr
 };
 //////////////////////////////////////////////////////
 SqlStmt memberdef_param_insert={
@@ -827,23 +827,23 @@ SqlStmt memberdef_param_insert={
     "( memberdef_id, param_id)"
   "VALUES "
     "(:memberdef_id,:param_id)"
-  ,NULL
+  ,nullptr
 };
 
 class TextGeneratorSqlite3Impl : public TextGeneratorIntf
 {
   public:
     TextGeneratorSqlite3Impl(StringVector &l) : m_list(l) { }
-    void writeString(const QCString & /*s*/,bool /*keepSpaces*/) const
+    void writeString(std::string_view /*s*/,bool /*keepSpaces*/) const override
     {
     }
-    void writeBreak(int) const
+    void writeBreak(int) const override
     {
       DBG_CTX(("writeBreak\n"));
     }
     void writeLink(const QCString & /*extRef*/,const QCString &file,
-                   const QCString &anchor,const QCString & /*text*/
-                  ) const
+                   const QCString &anchor,std::string_view /*text*/
+                  ) const override
     {
       std::string rs = file.str();
       if (!anchor.isEmpty())
@@ -901,7 +901,7 @@ static int step(SqlStmt &s,bool getRowId=FALSE, bool select=FALSE)
     return -1;
   }
   if (getRowId && select) rowid = sqlite3_column_int(s.stmt, 0); // works on selects, doesn't on inserts
-  if (getRowId && !select) rowid = sqlite3_last_insert_rowid(s.db); //works on inserts, doesn't on selects
+  if (getRowId && !select) rowid = static_cast<int>(sqlite3_last_insert_rowid(s.db)); //works on inserts, doesn't on selects
   sqlite3_reset(s.stmt);
   sqlite3_clear_bindings(s.stmt); // XXX When should this really be called
   return rowid;
@@ -910,7 +910,7 @@ static int step(SqlStmt &s,bool getRowId=FALSE, bool select=FALSE)
 static int insertPath(QCString name, bool local=TRUE, bool found=TRUE, int type=1)
 {
   int rowid=-1;
-  if (name==0) return rowid;
+  if (name==nullptr) return rowid;
 
   name = stripFromPath(name);
 
@@ -1028,7 +1028,7 @@ static void insertMemberFunctionParams(int memberdef_id, const MemberDef *md, co
     for (const Argument &a : declAl)
     {
       //const Argument *defArg = defAli.current();
-      const Argument *defArg = 0;
+      const Argument *defArg = nullptr;
       if (defIt!=defAl.end())
       {
         defArg = &(*defIt);
@@ -1122,7 +1122,7 @@ static void associateMember(const MemberDef *md, struct Refid member_refid, stru
 {
   // TODO: skip EnumValue only to guard against recording refids and member records
   // for enumvalues until we can support documenting them as entities.
-  if (md->memberType()==MemberType_EnumValue) return;
+  if (md->memberType()==MemberType::EnumValue) return;
   if (!md->isAnonymous()) // skip anonymous members
   {
     bindIntParameter(member_insert, ":scope_rowid", scope_refid.rowid);
@@ -1148,12 +1148,11 @@ static void stripQualifiers(QCString &typeStr)
 
 static int prepareStatement(sqlite3 *db, SqlStmt &s)
 {
-  int rc;
-  rc = sqlite3_prepare_v2(db,s.query,-1,&s.stmt,0);
+  int rc = sqlite3_prepare_v2(db,s.query,-1,&s.stmt,nullptr);
   if (rc!=SQLITE_OK)
   {
     err("prepare failed for:\n  %s\n  %s\n", s.query, sqlite3_errmsg(db));
-    s.db = NULL;
+    s.db = nullptr;
     return -1;
   }
   s.db = db;
@@ -1194,33 +1193,32 @@ static int prepareStatements(sqlite3 *db)
 
 static void beginTransaction(sqlite3 *db)
 {
-  char * sErrMsg = 0;
-  sqlite3_exec(db, "BEGIN TRANSACTION", NULL, NULL, &sErrMsg);
+  char * sErrMsg = nullptr;
+  sqlite3_exec(db, "BEGIN TRANSACTION", nullptr, nullptr, &sErrMsg);
 }
 
 static void endTransaction(sqlite3 *db)
 {
-  char * sErrMsg = 0;
-  sqlite3_exec(db, "END TRANSACTION", NULL, NULL, &sErrMsg);
+  char * sErrMsg = nullptr;
+  sqlite3_exec(db, "END TRANSACTION", nullptr, nullptr, &sErrMsg);
 }
 
 static void pragmaTuning(sqlite3 *db)
 {
-  char * sErrMsg = 0;
-  sqlite3_exec(db, "PRAGMA synchronous = OFF", NULL, NULL, &sErrMsg);
-  sqlite3_exec(db, "PRAGMA journal_mode = MEMORY", NULL, NULL, &sErrMsg);
-  sqlite3_exec(db, "PRAGMA temp_store = MEMORY;", NULL, NULL, &sErrMsg);
+  char * sErrMsg = nullptr;
+  sqlite3_exec(db, "PRAGMA synchronous = OFF", nullptr, nullptr, &sErrMsg);
+  sqlite3_exec(db, "PRAGMA journal_mode = MEMORY", nullptr, nullptr, &sErrMsg);
+  sqlite3_exec(db, "PRAGMA temp_store = MEMORY;", nullptr, nullptr, &sErrMsg);
 }
 
 static int initializeTables(sqlite3* db)
 {
-  int rc;
   msg("Initializing DB schema (tables)...\n");
   for (unsigned int k = 0; k < sizeof(table_schema) / sizeof(table_schema[0]); k++)
   {
     const char *q = table_schema[k][1];
-    char *errmsg;
-    rc = sqlite3_exec(db, q, NULL, NULL, &errmsg);
+    char *errmsg = nullptr;
+    int rc = sqlite3_exec(db, q, nullptr, nullptr, &errmsg);
     if (rc != SQLITE_OK)
     {
       err("failed to execute query: %s\n\t%s\n", q, errmsg);
@@ -1232,13 +1230,12 @@ static int initializeTables(sqlite3* db)
 
 static int initializeViews(sqlite3* db)
 {
-  int rc;
   msg("Initializing DB schema (views)...\n");
   for (unsigned int k = 0; k < sizeof(view_schema) / sizeof(view_schema[0]); k++)
   {
     const char *q = view_schema[k][1];
-    char *errmsg;
-    rc = sqlite3_exec(db, q, NULL, NULL, &errmsg);
+    char *errmsg = nullptr;
+    int rc = sqlite3_exec(db, q, nullptr, nullptr, &errmsg);
     if (rc != SQLITE_OK)
     {
       err("failed to execute query: %s\n\t%s\n", q, errmsg);
@@ -1288,6 +1285,19 @@ static void writeInnerConcepts(const ConceptLinkedRefMap &cl, struct Refid outer
   }
 }
 
+static void writeInnerModules(const ModuleLinkedRefMap &ml, struct Refid outer_refid)
+{
+  for (const auto &mod : ml)
+  {
+    struct Refid inner_refid = insertRefid(mod->getOutputFileBase());
+
+    bindIntParameter(contains_insert,":inner_rowid", inner_refid.rowid);
+    bindIntParameter(contains_insert,":outer_rowid", outer_refid.rowid);
+    step(contains_insert);
+  }
+}
+
+
 static void writeInnerPages(const PageLinkedRefMap &pl, struct Refid outer_refid)
 {
   for (const auto &pd : pl)
@@ -1413,12 +1423,12 @@ QCString getSQLDocBlock(const Definition *scope,
                 *parser.get(),
                 fileName,
                 lineNr,
-                const_cast<Definition*>(scope),
+                scope,
                 toMemberDef(def),
                 doc,
                 FALSE,
                 FALSE,
-                0,
+                QCString(),
                 FALSE,
                 FALSE,
                 Config_getBool(MARKDOWN_SUPPORT))
@@ -1466,7 +1476,7 @@ The XML schema claims:
     namespace page protocol service singleton struct type union
 
 Member kind comes from MemberDef::memberTypeName()
-  types.h defines 14 MemberType_*s
+  types.h defines 14 MemberType::*s
     _DCOP _Define _Enumeration _EnumValue _Event _Friend _Function _Interface
     _Property _Service _Signal _Slot _Typedef _Variable
       - xml doesn't include enumvalue here
@@ -1543,7 +1553,7 @@ static void generateSqlite3ForMember(const MemberDef *md, struct Refid scope_ref
   // - call graph
 
   // enum values are written as part of the enum
-  if (md->memberType()==MemberType_EnumValue) return;
+  if (md->memberType()==MemberType::EnumValue) return;
   if (md->isHidden()) return;
 
   QCString memType;
@@ -1639,11 +1649,11 @@ static void generateSqlite3ForMember(const MemberDef *md, struct Refid scope_ref
   bool isFunc=FALSE;
   switch (md->memberType())
   {
-    case MemberType_Function: // fall through
-    case MemberType_Signal:   // fall through
-    case MemberType_Friend:   // fall through
-    case MemberType_DCOP:     // fall through
-    case MemberType_Slot:
+    case MemberType::Function: // fall through
+    case MemberType::Signal:   // fall through
+    case MemberType::Friend:   // fall through
+    case MemberType::DCOP:     // fall through
+    case MemberType::Slot:
       isFunc=TRUE;
       break;
     default:
@@ -1666,7 +1676,7 @@ static void generateSqlite3ForMember(const MemberDef *md, struct Refid scope_ref
     bindIntParameter(memberdef_insert,":virt",static_cast<int>(md->virtualness()));
   }
 
-  if (md->memberType() == MemberType_Variable)
+  if (md->memberType() == MemberType::Variable)
   {
     bindIntParameter(memberdef_insert,":mutable",md->isMutable());
     bindIntParameter(memberdef_insert,":initonly",md->isInitonly());
@@ -1687,7 +1697,7 @@ static void generateSqlite3ForMember(const MemberDef *md, struct Refid scope_ref
       bindTextParameter(memberdef_insert,":bitfield",bitfield.stripWhiteSpace());
     }
   }
-  else if (md->memberType() == MemberType_Property)
+  else if (md->memberType() == MemberType::Property)
   {
     bindIntParameter(memberdef_insert,":readable",md->isReadable());
     bindIntParameter(memberdef_insert,":writable",md->isWritable());
@@ -1713,7 +1723,7 @@ static void generateSqlite3ForMember(const MemberDef *md, struct Refid scope_ref
     bindTextParameter(memberdef_insert,":read",md->getReadAccessor());
     bindTextParameter(memberdef_insert,":write",md->getWriteAccessor());
   }
-  else if (md->memberType() == MemberType_Event)
+  else if (md->memberType() == MemberType::Event)
   {
     bindIntParameter(memberdef_insert,":addable",md->isAddable());
     bindIntParameter(memberdef_insert,":removable",md->isRemovable());
@@ -1733,11 +1743,11 @@ static void generateSqlite3ForMember(const MemberDef *md, struct Refid scope_ref
   }
 
   // + declaration/definition arg lists
-  if (md->memberType()!=MemberType_Define &&
-      md->memberType()!=MemberType_Enumeration
+  if (md->memberType()!=MemberType::Define &&
+      md->memberType()!=MemberType::Enumeration
      )
   {
-    if (md->memberType()!=MemberType_Typedef)
+    if (md->memberType()!=MemberType::Typedef)
     {
       writeMemberTemplateLists(md);
     }
@@ -1831,7 +1841,7 @@ static void generateSqlite3ForMember(const MemberDef *md, struct Refid scope_ref
   {
     insertMemberFunctionParams(memberdef_id,md,def);
   }
-  else if (md->memberType()==MemberType_Define &&
+  else if (md->memberType()==MemberType::Define &&
           !md->argsString().isEmpty())
   {
     insertMemberDefineParams(memberdef_id,md,def);
@@ -1858,13 +1868,13 @@ static void generateSqlite3Section( const Definition *d,
                       const QCString & /*header*/=QCString(),
                       const QCString & /*documentation*/=QCString())
 {
-  if (ml==0) return;
+  if (ml==nullptr) return;
   for (const auto &md : *ml)
   {
     // TODO: necessary? just tracking what xmlgen does; xmlgen says:
     // namespace members are also inserted in the file scope, but
     // to prevent this duplication in the XML output, we filter those here.
-    if (d->definitionType()!=Definition::TypeFile || md->getNamespaceDef()==0)
+    if (d->definitionType()!=Definition::TypeFile || md->getNamespaceDef()==nullptr)
     {
       generateSqlite3ForMember(md, scope_refid, d);
     }
@@ -1911,7 +1921,7 @@ static void generateSqlite3ForClass(const ClassDef *cd)
   if (cd->isReference())        return; // skip external references.
   if (cd->isHidden())           return; // skip hidden classes.
   if (cd->isAnonymous())        return; // skip anonymous compounds.
-  if (cd->templateMaster()!=0)  return; // skip generated template instances.
+  if (cd->templateMaster()!=nullptr)  return; // skip generated template instances.
 
   struct Refid refid = insertRefid(cd->getOutputFileBase());
 
@@ -2025,7 +2035,7 @@ static void generateSqlite3ForClass(const ClassDef *cd)
   // this is just a list of *local* members
   for (const auto &ml : cd->getMemberLists())
   {
-    if ((ml->listType()&MemberListType_detailedLists)==0)
+    if (!ml->listType().isDetailed())
     {
       generateSqlite3Section(cd,ml.get(),refid,"user-defined");
     }
@@ -2059,6 +2069,61 @@ static void generateSqlite3ForConcept(const ConceptDef *cd)
   writeTemplateList(cd);
 }
 
+static void generateSqlite3ForModule(const ModuleDef *mod)
+{
+  // + contained class definitions
+  // + contained concept definitions
+  // + member groups
+  // + normal members
+  // + brief desc
+  // + detailed desc
+  // + location (file_id, line, column)
+  // - exports
+  // + used files
+
+  if (mod->isReference() || mod->isHidden()) return;
+  struct Refid refid = insertRefid(mod->getOutputFileBase());
+  if(!refid.created && compounddefExists(refid)){return;}
+  bindIntParameter(compounddef_insert,":rowid", refid.rowid);
+  bindTextParameter(compounddef_insert,":name",mod->name());
+  bindTextParameter(compounddef_insert,":kind","module");
+
+  int file_id = insertPath(mod->getDefFileName());
+  bindIntParameter(compounddef_insert,":file_id",file_id);
+  bindIntParameter(compounddef_insert,":line",mod->getDefLine());
+  bindIntParameter(compounddef_insert,":column",mod->getDefColumn());
+
+  getSQLDesc(compounddef_insert,":briefdescription",mod->briefDescription(),mod);
+  getSQLDesc(compounddef_insert,":detaileddescription",mod->documentation(),mod);
+
+  step(compounddef_insert);
+
+  // + contained class definitions
+  writeInnerClasses(mod->getClasses(),refid);
+
+  // + contained concept definitions
+  writeInnerConcepts(mod->getConcepts(),refid);
+
+  // + member groups
+  for (const auto &mg : mod->getMemberGroups())
+  {
+    generateSqlite3Section(mod,&mg->members(),refid,"user-defined",mg->header(),
+        mg->documentation());
+  }
+
+  // + normal members
+  for (const auto &ml : mod->getMemberLists())
+  {
+    if (ml->listType().isDeclaration())
+    {
+      generateSqlite3Section(mod,ml.get(),refid,"user-defined");
+    }
+  }
+
+  // + files
+  writeInnerFiles(mod->getUsedFiles(),refid);
+}
+
 // kinds: constants library module namespace package
 static void generateSqlite3ForNamespace(const NamespaceDef *nd)
 {
@@ -2109,7 +2174,7 @@ static void generateSqlite3ForNamespace(const NamespaceDef *nd)
   // + normal members
   for (const auto &ml : nd->getMemberLists())
   {
-    if ((ml->listType()&MemberListType_declarationLists)!=0)
+    if (ml->listType().isDeclaration())
     {
       generateSqlite3Section(nd,ml.get(),refid,"user-defined");
     }
@@ -2157,8 +2222,9 @@ static void generateSqlite3ForFile(const FileDef *fd)
   for (const auto &ii : fd->includeFileList())
   {
     int src_id=insertPath(fd->absFilePath(),!fd->isReference());
-    int dst_id;
+    int dst_id=0;
     QCString dst_path;
+    bool isLocal = (ii.kind & IncludeKind_LocalMask)!=0;
 
     if(ii.fileDef) // found file
     {
@@ -2173,16 +2239,16 @@ static void generateSqlite3ForFile(const FileDef *fd)
       {
         dst_path = ii.fileDef->absFilePath();
       }
-      dst_id = insertPath(dst_path,ii.local);
+      dst_id = insertPath(dst_path,isLocal);
     }
     else // can't find file
     {
-      dst_id = insertPath(ii.includeName,ii.local,FALSE);
+      dst_id = insertPath(ii.includeName,isLocal,FALSE);
     }
 
     DBG_CTX(("-----> FileDef includeInfo for %s\n", qPrint(ii.includeName)));
-    DBG_CTX(("       local:    %d\n", ii.local));
-    DBG_CTX(("       imported: %d\n", ii.imported));
+    DBG_CTX(("       local:    %d\n", isLocal));
+    DBG_CTX(("       imported: %d\n", (ii.kind & IncludeKind_ImportMask)!=0));
     if(ii.fileDef)
     {
       DBG_CTX(("include: %s\n", qPrint(ii.fileDef->absFilePath())));
@@ -2190,11 +2256,11 @@ static void generateSqlite3ForFile(const FileDef *fd)
     DBG_CTX(("       src_id  : %d\n", src_id));
     DBG_CTX(("       dst_id: %d\n", dst_id));
 
-    bindIntParameter(incl_select,":local",ii.local);
+    bindIntParameter(incl_select,":local",isLocal);
     bindIntParameter(incl_select,":src_id",src_id);
     bindIntParameter(incl_select,":dst_id",dst_id);
     if (step(incl_select,TRUE,TRUE)==0) {
-      bindIntParameter(incl_insert,":local",ii.local);
+      bindIntParameter(incl_insert,":local",isLocal);
       bindIntParameter(incl_insert,":src_id",src_id);
       bindIntParameter(incl_insert,":dst_id",dst_id);
       step(incl_insert);
@@ -2205,8 +2271,9 @@ static void generateSqlite3ForFile(const FileDef *fd)
   for (const auto &ii : fd->includedByFileList())
   {
     int dst_id=insertPath(fd->absFilePath(),!fd->isReference());
-    int src_id;
+    int src_id=0;
     QCString src_path;
+    bool isLocal = (ii.kind & IncludeKind_LocalMask)!=0;
 
     if(ii.fileDef) // found file
     {
@@ -2221,18 +2288,18 @@ static void generateSqlite3ForFile(const FileDef *fd)
       {
         src_path = ii.fileDef->absFilePath();
       }
-      src_id = insertPath(src_path,ii.local);
+      src_id = insertPath(src_path,isLocal);
     }
     else // can't find file
     {
-      src_id = insertPath(ii.includeName,ii.local,FALSE);
+      src_id = insertPath(ii.includeName,isLocal,FALSE);
     }
 
-    bindIntParameter(incl_select,":local",ii.local);
+    bindIntParameter(incl_select,":local",isLocal);
     bindIntParameter(incl_select,":src_id",src_id);
     bindIntParameter(incl_select,":dst_id",dst_id);
     if (step(incl_select,TRUE,TRUE)==0) {
-      bindIntParameter(incl_insert,":local",ii.local);
+      bindIntParameter(incl_insert,":local",isLocal);
       bindIntParameter(incl_insert,":src_id",src_id);
       bindIntParameter(incl_insert,":dst_id",dst_id);
       step(incl_insert);
@@ -2258,7 +2325,7 @@ static void generateSqlite3ForFile(const FileDef *fd)
   // + normal members
   for (const auto &ml : fd->getMemberLists())
   {
-    if ((ml->listType()&MemberListType_declarationLists)!=0)
+    if (ml->listType().isDeclaration())
     {
       generateSqlite3Section(fd,ml.get(),refid,"user-defined");
     }
@@ -2309,6 +2376,9 @@ static void generateSqlite3ForGroup(const GroupDef *gd)
   // + concepts
   writeInnerConcepts(gd->getConcepts(),refid);
 
+  // + modules
+  writeInnerModules(gd->getModules(),refid);
+
   // + namespaces
   writeInnerNamespaces(gd->getNamespaces(),refid);
 
@@ -2328,7 +2398,7 @@ static void generateSqlite3ForGroup(const GroupDef *gd)
   // + members
   for (const auto &ml : gd->getMemberLists())
   {
-    if ((ml->listType()&MemberListType_declarationLists)!=0)
+    if (ml->listType().isDeclaration())
     {
       generateSqlite3Section(gd,ml.get(),refid,"user-defined");
     }
@@ -2460,14 +2530,13 @@ static sqlite3* openDbConnection()
 {
 
   QCString outputDirectory = Config_getString(SQLITE3_OUTPUT);
-  sqlite3 *db;
-  int rc;
+  sqlite3 *db = nullptr;
 
-  rc = sqlite3_initialize();
+  int rc = sqlite3_initialize();
   if (rc != SQLITE_OK)
   {
     err("sqlite3_initialize failed\n");
-    return NULL;
+    return nullptr;
   }
 
   std::string dbFileName = "doxygen_sqlite3.db";
@@ -2482,7 +2551,7 @@ static sqlite3* openDbConnection()
     else
     {
       err("doxygen_sqlite3.db already exists! Rename, remove, or archive it to regenerate\n");
-      return NULL;
+      return nullptr;
     }
   }
 
@@ -2490,7 +2559,7 @@ static sqlite3* openDbConnection()
     fi.absFilePath().c_str(),
     &db,
     SQLITE_OPEN_READWRITE | SQLITE_OPEN_CREATE,
-    0
+    nullptr
   );
   if (rc != SQLITE_OK)
   {
@@ -2510,17 +2579,15 @@ void generateSqlite3()
   // + related pages
   // + examples
   // + main page
-  sqlite3 *db;
-
-  db = openDbConnection();
-  if (db==NULL)
+  sqlite3 *db = openDbConnection();
+  if (db==nullptr)
   {
     return;
   }
 
 # ifdef SQLITE3_DEBUG
   // debug: show all executed statements
-  sqlite3_trace(db, &sqlLog, NULL);
+  sqlite3_trace(db, &sqlLog, nullptr);
 # endif
 
   beginTransaction(db);
@@ -2551,6 +2618,13 @@ void generateSqlite3()
     generateSqlite3ForConcept(cd.get());
   }
 
+  // + modules
+  for (const auto &mod : ModuleManager::instance().modules())
+  {
+    msg("Generating Sqlite3 output for module %s\n",qPrint(mod->name()));
+    generateSqlite3ForModule(mod.get());
+  }
+
   // + namespaces
   for (const auto &nd : *Doxygen::namespaceLinkedMap)
   {
@@ -2611,10 +2685,4 @@ void generateSqlite3()
   endTransaction(db);
 }
 
-#else // USE_SQLITE3
-void generateSqlite3()
-{
-  err("sqlite3 support has not been compiled in!\n");
-}
-#endif
 // vim: noai:ts=2:sw=2:ss=2:expandtab
